﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Windows.Forms;
using Gaudeix.Admin.Entities.Menu;
using Gaudeix.Admin.Infrastructure.Factory;
using Gaudeix.Admin.Infrastructure.Forms;
using Gaudeix.Admin.Resources;
using Gaudeix.Admin.Utils;
using Gaudeix.Admin.WCF.Services.Entitats;
using Gaudeix.Admin.WCF.Services.TaulesBase;

namespace Gaudeix.Admin.Forms.Entitats
{
    public partial class DetailEntitat : Form, IChildForm
    {
        #region Variables privades
        private const string MENU_TANCAR = "Tancar";
        private const string MENU_GUARDAR_CANVIS = "GuardarCanvis";
        private const string MENU_ESBORRAR = "Esborrar";
        private int IdEmpresa = -1;

        #endregion

        #region Constructors
        /// <summary>
        /// Constructor que inicialitza el formulari carregant una entitat existent
        /// </summary>
        /// <param name="idEntitat">Id d'entitat a carregar</param>
        public DetailEntitat(int idEntitat)
            : this()
        {
            this.IdEmpresa = idEntitat;
            this.CarregarEntitat(this.IdEmpresa);
        }

        /// <summary>
        /// Constructor. Inicialitza el formulari
        /// </summary>
        public DetailEntitat()
        {
            InitializeComponent();
            InitializeControls();
        }
        #endregion

        #region Mètodes interns
        /// <summary>
        /// Carrega les dades del codi d'entitat passat com a paràmetre
        /// </summary>
        /// <param name="idEntitat">Codi de l'entitat a carregar</param>
        private void CarregarEntitat(int idEntitat)
        {
            EntitatsClient entitats = null;
            TaulesBaseClient taulesBase = null;
            try
            {
                entitats = WcfFactory.GetEntitatsClient();
                DetallEmpresa empresa = entitats.GetDetallEmpresa(idEntitat);

                // Mostrem les dades generals de l'empresa
                this.txtAdreca.Text = empresa.Adreca;
                this.txtCIF.Text = empresa.Cif;
                this.txtContactes.Text = empresa.Contactes;
                this.txtEmail.Text = empresa.Email;
                this.txtNom.Text = empresa.Nom;
                this.cboProvincia.SelectedValue = empresa.IdProvincia;
                if (empresa.IdProvincia > 0)
                {
                    taulesBase = WcfFactory.GetTaulesBaseClient();
                    FormUtils.CarregaCombo(this.cboPoblacio,
                        taulesBase.GetLlistaPoblacionsByProvincia(empresa.IdProvincia));
                    cboPoblacio.SelectedValue = empresa.IdPoblacio;
                }
                // Carreguem la llista d'ofertes
                this.CarregarOfertes(empresa.Ofertes);
            }
            catch (FaultException ex)
            {
                FormUtils.MissatgeError(CommonResources.ErrorsCarregar + "\n" + ex.Message);
                return;
            }
            finally
            {
                if (entitats != null) { entitats.Close(); }
                if (taulesBase != null) { taulesBase.Close(); }
            }
        }

        /// <summary>
        /// Inicialitza els controls del formulari
        /// </summary>
        private void InitializeControls()
        {
            // Carreguem el combo de provincies
            FormUtils.CarregaCombo(this.cboProvincia, WcfUtils.GetProvincies());
            titolMain.Image = CommonResources.IconEmpreses.ToBitmap();
        }

        /// <summary>
        /// Guarda les dades de l'empresa a la base de dades, a través d'una crida WCF
        /// </summary>
        private void GuardarEmpresa()
        {
            if (!FormUtils.MissatgeConfirmacio(CommonResources.ConfirmarGuardar))
            {
                return;
            }
            DetallEmpresa empresa = new DetallEmpresa();

            empresa.Id = this.IdEmpresa;
            empresa.Nom = this.txtNom.Text;
            empresa.Cif = this.txtCIF.Text;
            empresa.Adreca = this.txtAdreca.Text;
            empresa.Email = this.txtEmail.Text;
            if (cboPoblacio.SelectedItem != null)
            {
                empresa.IdPoblacio = ((ItemIdDescripcio)cboPoblacio.SelectedItem).Id;
            }
            empresa.Contactes = this.txtContactes.Text;
            EntitatsClient client = null;
            try
            {
                client = WcfFactory.GetEntitatsClient();
                client.GuardarEntitat(empresa);
            }
            catch (FaultException f)
            {
                FormUtils.MissatgeError(CommonResources.ErrorsTrobatGuardar + "\n" + f.Message);
                return;
            }
            finally
            {
                if (client != null) { client.Close(); }
            }
            FormUtils.MissatgeOperacioOk(CommonResources.InfoDadesGuardades);
        }

        /// <summary>
        /// Esborra l'empresa actual de la base de dades
        /// </summary>
        private void EsborrarEntitat()
        {
            EntitatsClient client = null;
            try
            {
                if (!FormUtils.MissatgeConfirmacio(CommonResources.ConfirmarEliminacio))
                {
                    return;
                }
                client = WcfFactory.GetEntitatsClient();
                client.EsborrarEmpreses(new int[] { this.IdEmpresa });
            }
            catch (FaultException f)
            {
                FormUtils.MissatgeError(CommonResources.ErrorsTrobatGuardar + "\n" + f.Message);
                return;
            }
            finally
            {
                if (client != null) { client.Close(); }
            }
            FormUtils.MissatgeOperacioOk(CommonResources.InfoRegistreEliminat);
        }

        /// <summary>
        /// Carrega les ofertes de l'empresa a la llista inferior de la pantalla
        /// </summary>
        /// <param name="ofertes">Llista d'ofertes a carregar</param>
        private void CarregarOfertes(ItemOfertesEmpresa[] ofertes)
        {
            this.lsvOfertes.Items.Clear();
            if (ofertes == null)
            {
                return;
            }
            ListViewItem item;
            foreach (ItemOfertesEmpresa oferta in ofertes)
            {
                item = lsvOfertes.Items.Add(oferta.Titol);
                item.SubItems.Add(FormatUtils.FormatData(oferta.DataInici));
                item.SubItems.Add(FormatUtils.FormatData(oferta.DataFi));
                item.SubItems.Add(FormatUtils.FormatImport(oferta.PreuBase));
                item.SubItems.Add(FormatUtils.FormatImport(oferta.PreuVenda));
                item.SubItems.Add(oferta.NomProvincia);
                item.SubItems.Add(oferta.NomPoblacio);
            }
        }
        #endregion

        #region IChildForm Members

        public IList<ItemMenu> GetFormMenu()
        {
            IList<ItemMenu> resultat = new List<ItemMenu>();
            resultat.Add(new ItemMenu(CommonResources.IconSortir.ToBitmap(), "Tancar", MENU_TANCAR));
            resultat.Add(new ItemMenu(null, "", "", true));
            resultat.Add(new ItemMenu(CommonResources.IconGuardar.ToBitmap(), "Guardar canvis", MENU_GUARDAR_CANVIS));
            if (this.IdEmpresa > 0)
            {
                resultat.Add(new ItemMenu(CommonResources.IconEsborrar.ToBitmap(), "Esborrar", MENU_ESBORRAR));
            }
            return resultat;
        }

        public void ExecuteCommand(string command)
        {
            switch (command)
            {
                case MENU_TANCAR:
                    this.Close();
                    break;
                case MENU_GUARDAR_CANVIS:
                    GuardarEmpresa();
                    break;
                case MENU_ESBORRAR:
                    EsborrarEntitat();
                    break;
            }
        }

        #endregion

        #region Control d'events
        private void cboProvincia_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Carreguem les poblacions en base al id de provincia seleccionat
            //int idProvincia = ((ItemIdDescripcio)cboProvincia.SelectedItem).Id;
            int idProvincia = (int)cboProvincia.SelectedValue;
            FormUtils.CarregaCombo(this.cboPoblacio, WcfUtils.GetPoblacionsByIdProvincia(idProvincia));
        }
        #endregion

    }
}
