﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Windows.Forms;
using Gaudeix.Admin.Entities.Menu;
using Gaudeix.Admin.Forms.Categories;
using Gaudeix.Admin.Forms.Entitats;
using Gaudeix.Admin.Infrastructure.Events;
using Gaudeix.Admin.Infrastructure.Factory;
using Gaudeix.Admin.Infrastructure.Forms;
using Gaudeix.Admin.Resources;
using Gaudeix.Admin.Utils;
using Gaudeix.Admin.WCF.Services.Ofertes;

namespace Gaudeix.Admin.Forms.Ofertes
{
    public partial class SearchOfertes : Form, IParentForm
    {
        #region Propietats privades
        private const string MENU_TANCAR = "Tancar";
        private const string MENU_EXECUTAR_CONSULTA = "ExecConsulta";
        private const string MENU_INSERIR = "Inserir";
        private const string MENU_ESBORRAR_SELECCIONATS = "EsborrarSeleccionades";

        private int IdEmpresa;
        private int[] IdCategories;
        #endregion

        #region Declaració events públics
        public event EventHandler<ChildFormEventArgs> ShowChildForm;
        #endregion

        #region Constructors
        public SearchOfertes()
        {
            InitializeComponent();
            InitializeControls();
        }

        #endregion

        #region IChildForm Members

        public IList<ItemMenu> GetFormMenu()
        {
            IList<ItemMenu> resultat = new List<ItemMenu>();
            resultat.Add(new ItemMenu(CommonResources.IconSortir.ToBitmap(), "Tancar", MENU_TANCAR));
            resultat.Add(new ItemMenu(null, "", "", true));
            resultat.Add(new ItemMenu(CommonResources.IconCercar.ToBitmap(), "Executar consulta", MENU_EXECUTAR_CONSULTA));
            resultat.Add(new ItemMenu(CommonResources.IconAfegir.ToBitmap(), "Inserir oferta", MENU_INSERIR));
            resultat.Add(new ItemMenu(CommonResources.IconEsborrar.ToBitmap(), "Esborrar seleccionades", MENU_ESBORRAR_SELECCIONATS));
            return resultat;
        }

        public void ExecuteCommand(string command)
        {
            switch (command)
            {
                case MENU_TANCAR:
                    this.Close();
                    break;
                case MENU_EXECUTAR_CONSULTA:
                    this.ExecutarConsulta();
                    break;
                case MENU_INSERIR:
                    this.ObrirDetall(0);
                    break;
                case MENU_ESBORRAR_SELECCIONATS:
                    this.EsborrarSeleccionats();
                    break;
            }
        }

        #endregion

        #region Mètodes privats
        /// <summary>
        /// Executa la consulta de cerca de registres
        /// </summary>
        private void ExecutarConsulta()
        {
            OfertesClient client = null;
            ItemLlistaOfertes[] ofertes;
            try
            {
                client = WcfFactory.GetOfertesClient();
                ofertes = client.ConsultarOfertes(this.CollectSearchParams());
            }
            catch (FaultException ex)
            {
                FormUtils.MissatgeError(CommonResources.ErrorsCarregar + "\n" + ex.Message);
                return;
            }
            finally
            {
                if (client != null) { client.Close(); }
            }
            this.CarregarOfertes(ofertes);
        }

        /// <summary>
        /// Inicialitza els controls de la pantalla
        /// </summary>
        private void InitializeControls()
        {
            // Carreguem el combo de provincies
            FormUtils.CarregaCombo(this.cboProvincia, WcfUtils.GetProvincies());
            this.titolMain.Image = CommonResources.IconOfertes.ToBitmap();
        }

        /// <summary>
        /// Carrega les ofertes a la pantalla
        /// </summary>
        /// <param name="llista">Llista d'ofertes a carregar</param>
        private void CarregarOfertes(ItemLlistaOfertes[] llista)
        {
            this.lsvResultats.Items.Clear();
            if (llista.Count() == 0)
            {
                FormUtils.MissatgeInformacio(CommonResources.InfoNoRegistres);
                return;
            }
            lsvResultats.BeginUpdate();

            ListViewItem item;
            foreach (ItemLlistaOfertes oferta in llista)
            {
                item = lsvResultats.Items.Add(String.Empty);
                item.Tag = oferta.IdOferta;
                item.SubItems.Add(oferta.NomEmpresa);
                item.SubItems.Add(oferta.Titol);
                item.SubItems.Add(FormatUtils.FormatImport(oferta.PreuBase));
                item.SubItems.Add(FormatUtils.FormatImport(oferta.PreuVenda));
                item.SubItems.Add(FormatUtils.FormatData(oferta.DataInici));
                item.SubItems.Add(FormatUtils.FormatData(oferta.DataFi));
                item.SubItems.Add(oferta.NomCategoria);
            }
            lsvResultats.EndUpdate();
        }


        /// <summary>
        /// Recopila els paràmetres de cerca de la pantalla
        /// </summary>
        /// <returns>Paràmetres de cerca</returns>
        private ParamsSearchOfertes CollectSearchParams()
        {
            ParamsSearchOfertes result = new ParamsSearchOfertes();
            result.IdEmpresa = this.IdEmpresa;
            result.Titol = this.txtTitol.Text;
            result.PreuBaseInit = ParseUtils.ParseDecimal(this.txtPreuBaseDesDe.Text);
            result.PreuBaseFi = ParseUtils.ParseDecimal(this.txtPreuBaseFins.Text);
            result.PreuVendaInit = ParseUtils.ParseDecimal(this.txtPreuVendaDesDe.Text);
            result.PreuVendaFi = ParseUtils.ParseDecimal(this.txtPreuVendaFins.Text);
            result.DataIniciDesDe = ParseUtils.ParseDateTime(this.txtDataIniciDesDe.Text);
            result.DataIniciFins = ParseUtils.ParseDateTime(this.txtDataIniciFins.Text);
            result.DataFiDesDe = ParseUtils.ParseDateTime(this.txtDataFiDesDe.Text);
            result.DataFiFins = ParseUtils.ParseDateTime(this.txtDataFiFins.Text);
            result.idProvincia = (int)this.cboProvincia.SelectedValue;
            result.idPoblacio = (int)this.cboPoblacio.SelectedValue;
            result.idCategories = this.IdCategories;
            return result;
        }

        /// <summary>
        /// Obre el detall de la oferta
        /// </summary>
        /// <param name="idOferta">Codi de la oferta a obrir</param>
        private void ObrirDetall(int idOferta)
        {
            if (idOferta > 0)
            {
                this.OnShowChildForm(new ChildFormEventArgs(new DetailOferta(idOferta)));
            }
            else
            {
                this.OnShowChildForm(new ChildFormEventArgs(new DetailOferta()));
            }
        }

        /// <summary>
        /// Esborra els elements seleccionats de la llistaº
        /// </summary>
        private void EsborrarSeleccionats()
        {
            OfertesClient client = null;
            try
            {
                if (lsvResultats.CheckedItems.Count == 0)
                {
                    FormUtils.MissatgeError(CommonResources.SeleccioniElement);
                }
                if (!FormUtils.MissatgeConfirmacio(CommonResources.ConfirmarEliminacio))
                {
                    return;
                }
                IList<int> ids = new List<int>();
                foreach (ListViewItem item in lsvResultats.CheckedItems)
                {
                    ids.Add((int)item.Tag);
                }
                client = WcfFactory.GetOfertesClient();
                client.EsborrarOfertes(ids.ToArray<int>());
            }
            catch (FaultException ex)
            {
                FormUtils.MissatgeError(CommonResources.ErrorsOperacio + "\n" + ex.Message);
                return;
            }
            finally
            {
                if (client != null) { client.Close(); }
            }
        }
        #endregion

        #region Control d'events
        /// <summary>
        /// Evita que es dispari l'event si no hi ha cap controlador escoltant.
        /// </summary>
        /// <param name="e">Paràmetres de l'event</param>
        protected virtual void OnShowChildForm(ChildFormEventArgs e)
        {
            // Fem una copia temporal del event per a evitar la possibilitat de que 
            // el subscriptor anuli la subscripció quan es dispari l'event
            EventHandler<ChildFormEventArgs> handler = ShowChildForm;
            if (handler != null)
            {
                handler(this, e);
            }
        }

        #endregion

        #region Control dels events del formulari
        private void cboProvincia_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Carreguem les poblacions en base al id de provincia seleccionat
            int idProvincia = (int)cboProvincia.SelectedValue;
            FormUtils.CarregaCombo(this.cboPoblacio, WcfUtils.GetPoblacionsByIdProvincia(idProvincia));
        }

        private void btnEmpresa_Click(object sender, EventArgs e)
        {
            PopUpSearchEntitat entitat = new PopUpSearchEntitat();
            entitat.SelectOnlyOne = true;
            if (entitat.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                this.IdEmpresa = entitat.selectedIds[0];
                this.txtEmpresa.Text = entitat.selectedNames[0];
            }
        }

        private void btnCategories_Click(object sender, EventArgs e)
        {
            PopUpSearchCategories categories = new PopUpSearchCategories();
            categories.selectedIds = this.IdCategories;
            if (categories.ShowDialog(this) == DialogResult.OK)
            {
                this.IdCategories = categories.selectedIds;
                this.txtCategories.Text = String.Join(", ", categories.selectedNames);
            }
        }

        private void lsvResultats_DoubleClick(object sender, EventArgs e)
        {
            if (lsvResultats.SelectedItems.Count > 0)
            {
                int idOferta = (int)lsvResultats.SelectedItems[0].Tag;
                this.ObrirDetall(idOferta);
            }
        }
        #endregion
    }
}
