﻿using System;
using System.Configuration;
using Gaudeix.Admin.Constants;


namespace Gaudeix.Admin.Infrastructure.Config
{
    /// <summary>
    /// Classe d'utilitats per a gestionar la configuració interna del programa
    /// </summary>
    public static class ConfigManager
    {
        #region Propietats públiques
        // Usuari connectat
        public static string CurrentLoginUser { get; private set; }
        // Password de l'usuari connectat
        public static string CurrentLoginPassword { get; private set; }
        #endregion

        #region Mètodes públics
        /// <summary>
        ///  Obté l'adreça base dels endpoints
        /// </summary>
        /// <returns>Adreça base dels endpoints</returns>
        public static string GetBaseAddress()
        {
            return AdminSettings.Default.BaseAddress;
        }

        /// <summary>
        /// Retorna el darrer nom d'usuari que va iniciar sessió al sistema
        /// </summary>
        /// <returns>Darrer usuari identificat</returns>
        public static string GetLastUserName()
        {
            return AdminSettings.Default.LastUser;
        }

        /// <summary>
        /// Obté un endpoint per a operacions sobre entitats
        /// </summary>
        /// <returns>Endpoint sol·licitat</returns>
        public static string GetEndpointEntitats()
        {
            return GetEndpoint(new Uri(GetBaseAddress()), CommonConstants.SERVICE_PATH_ENTITATS);
        }

        /// <summary>
        /// Obté un endpoint per a operacions sobre compres 
        /// </summary>
        /// <returns>Endpoint sol·licitat</returns>
        public static string GetEndpointCompres()
        {
            return GetEndpoint(new Uri(GetBaseAddress()), CommonConstants.SERVICE_PATH_COMPRES);
        }

        /// <summary>
        /// Obté un endpoint per a operacions sobre emails
        /// </summary>
        /// <returns>Endpoint sol·licitat</returns>
        public static string GetEndpointEmail()
        {
            return GetEndpoint(new Uri(GetBaseAddress()), CommonConstants.SERVICE_PATH_EMAIL);
        }

        /// <summary>
        /// Obté un endpoint per a operacions sobre ofertes 
        /// </summary>
        /// <returns>Endpoint sol·licitat</returns>
        public static string GetEndpointOfertes()
        {
            return GetEndpoint(new Uri(GetBaseAddress()), CommonConstants.SERVICE_PATH_OFERTES);
        }

        /// <summary>
        /// Obté un endpoint per a operacions sobre taules base 
        /// </summary>
        /// <returns>Endpoint sol·licitat</returns>
        public static string GetEndpointTaulesBase()
        {
            return GetEndpoint(new Uri(GetBaseAddress()), CommonConstants.SERVICE_PATH_TAULES_BASE);
        }

        /// <summary>
        /// Obté un endpoint per a operacions sobre usuaris 
        /// </summary>
        /// <returns>Endpoint sol·licitat</returns>
        public static string GetEndpointUsuaris()
        {
            return GetEndpoint(new Uri(GetBaseAddress()), CommonConstants.SERVICE_PATH_USUARIS);
        }

        /// <summary>
        /// Obté un endpoint per a operacions sobre estadistiques 
        /// </summary>
        /// <returns>Endpoint sol·licitat</returns>
        public static string GetEndpointEstadistiques()
        {
            return GetEndpoint(new Uri(GetBaseAddress()), CommonConstants.SERVICE_PATH_ESTADISTIQUES);
        }

        /// <summary>
        /// Guarda les dades relatives a l'inici de sessió actual
        /// </summary>
        /// <param name="username">Nom d'usuari</param>
        /// <param name="password">Paraula clau</param>
        public static void SetCurrentLogin(string username, string password)
        {
            AdminSettings.Default.LastUser = username;
            AdminSettings.Default.Save();
            CurrentLoginUser = username;
            CurrentLoginPassword = password;
        }

        /// <summary>
        /// Actualitza les adreces dels endpoints a partir d'una nova adreça base
        /// </summary>
        /// <param name="newBaseAddress"></param>
        public static void UpdateEndpointsAddress(string newBaseAddress)
        {
            // Obrim el fitxer de configuració
            if (GetBaseAddress().Equals(newBaseAddress))
            {
                return;
            }
            // Actualitzem el valor de l'adreça base
            AdminSettings.Default.BaseAddress = newBaseAddress;

            // Carreguem la URI a partir de la nova adreça base
            Uri baseUri = new Uri(newBaseAddress);

            // Actualitzem l'adreça del endpoint de compres
            AdminSettings.Default.EndpointCompres = GetEndpoint(baseUri, CommonConstants.SERVICE_PATH_COMPRES);

            // Actualitzem l'adreça del endpoint de correus electrònics
            AdminSettings.Default.EndpointEmail = GetEndpoint(baseUri, CommonConstants.SERVICE_PATH_EMAIL);

            // Actualitzem l'adreça del endpoint d'entitats
            AdminSettings.Default.EndpointEntitats = GetEndpoint(baseUri, CommonConstants.SERVICE_PATH_ENTITATS);

            // Actualitzem l'adreça del endpoint d'estadistiques
            AdminSettings.Default.EndpointEstadistiques = GetEndpoint(baseUri, CommonConstants.SERVICE_PATH_ESTADISTIQUES);

            // Actualitzem l'adreça del endpoint d'ofertes
            AdminSettings.Default.EndpointOfertes = GetEndpoint(baseUri, CommonConstants.SERVICE_PATH_OFERTES);

            // Actualitzem l'adreça del endpoint de taules base
            AdminSettings.Default.EndpointTaulesBase = GetEndpoint(baseUri, CommonConstants.SERVICE_PATH_TAULES_BASE);

            // Actualitzem l'adreça del endpoint d'usuaris
            AdminSettings.Default.EndpointUsuaris = GetEndpoint(baseUri, CommonConstants.SERVICE_PATH_USUARIS);

            AdminSettings.Default.Save();
        }

        #endregion

        #region Mètodes privats

        /// <summary>
        /// Obté la URL d'un endpoint determinat
        /// </summary>
        /// <param name="baseUri">Adreça base de l'aplicació</param>
        /// <param name="servicePath">Ruta relativa del servei</param>
        /// <returns>URL del endpoint</returns>
        private static string GetEndpoint(Uri baseUri, string servicePath)
        {
            String url = String.Format("{0}://{1}{2}{3}", baseUri.Scheme, baseUri.Authority, baseUri.LocalPath, servicePath);
            return new Uri(url).ToString();
        }
        #endregion
    }
}
