﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Gaudeix.Common.Infrastructure.Entity;
using Castle.Core.Logging;

namespace Gaudeix.Common.Infrastructure.Dao
{
    /// <summary>
    /// Interficie de definició de les operacions a implementar pel Dao base
    /// </summary>
    /// <typeparam name="TEntity">Tipus de l'entitat que gestiona el DAO. 
    /// Es requereix que heredi de la entitat BaseEntity, amb el que ha de tractar-se 
    /// d'una entitat mapejada de NHibernate</typeparam>
    /// <typeparam name="TKey">Tipus de la clau primària (PK) de l'entitat de NHibernate.
    /// Es requereix que implementi la interficie IComparable.</typeparam>
    /// <seealso cref="Gaudeix.Common.Infrastructure.Entity.BaseEntity"/>
    /// <seealso cref="System.IComparable"/>
    public interface IBaseDao<TEntity, TKey> : IDao
        where TEntity : BaseEntity
        where TKey : IComparable
    {
        #region Propietats a implementar

        /// <summary>
        /// Retorna o estableix el gestor de log per a l'entitat
        /// </summary>
        ILogger logger { get; set; }
        #endregion

        #region Mètodes a implementar
        /// <summary>
        /// Obté una entitat a partir de la seva clau primària
        /// </summary>
        /// <param name="id">Clau primària de l'entitat</param>
        /// <returns>Entitat identificada per la clau primària subministrada</returns>
        /// <exception cref="GaudeixDaoException">En cas d'error en l'accés a la BD</exception>
        TEntity GetById(TKey id);

        /// <summary>
        /// Guarda una nova entitat a la base de dades. Es requereix que la nova entitat no tingui la PK informada.
        /// </summary>
        /// <param name="entity">Nova entitat a guardar</param>
        /// <returns>PK associada a la BD.</returns>
        /// <exception cref="GaudeixDaoException">En cas d'error en l'accés a la BD</exception>
        TKey Save(TEntity entity);

        /// <summary>
        /// Guarda les dades de l'entitat passada com a paràmetre, inserint-la si no existeix, o actualitzant les seves dades si ja existia.
        /// </summary>
        /// <param name="entity">Entitat a guardar o actualitzar</param>
        /// <returns>Clau primària associada a l'entitat</returns>
        /// <exception cref="GaudeixDaoException">En cas d'error en l'accés a la BD</exception>
        TKey SaveOrUpdate(TEntity entity);

        /// <summary>
        /// Esborra una entitat a la base de dades
        /// </summary>
        /// <param name="entity">Entitat a esborrar</param>
        /// <exception cref="GaudeixDaoException">En cas d'error en l'accés a la BD</exception>
        void Delete(TEntity entity);

        /// <summary>
        /// Actualitza les dades d'una entitat a la base de dades. Es requereix que l'entitat ja existeixi previament.
        /// </summary>
        /// <param name="entity">Entitat a actualitzar (ha d'existir a la base de dades)</param>
        /// <returns>Clau primària associada a l'entitat</returns>
        /// <exception cref="GaudeixDaoException">En cas d'error en l'accés a la BD</exception>
        TKey Update(TEntity entity);
        #endregion
    }
}
