﻿using System.Collections.Generic;
using Gaudeix.Common.Infrastructure.Manager;
using Gaudeix.Common.Models.Browse;
using Gaudeix.Common.Models.Oferta;

namespace Gaudeix.Common.Interfaces.Manager
{
    /// <summary>
    /// Interficie de declaracio d'operacions sobre ofertes per a la capa de negoci
    /// </summary>
    public interface IOfertaManager : IBaseManager
    {
        #region Mètodes a implementar

        /// <summary>
        /// Retorna un llistat de les ofertes que compleixen els paràmetres de cerca.
        /// </summary>
        /// <returns>Llista d'ofertes que compleixen amb els paràmetres de cerca</returns>
        /// <exception cref="GaudeixBusinessException">En cas d'error durant l'execució del mètode</exception>
        IList<ItemLlistaOfertes> ConsultarOfertes(ParamsSearchOfertes parametres);

        /// <summary>
        /// Retorna un llistat de les ofertes que compleixen els paràmetres de cerca, però fent servir paginació de registres
        /// </summary>
        /// <param name="parametres">Paràmetres de cerca</param>
        /// <param name="numPlana">Número de plana a recuperar</param>
        /// <param name="numRegsPlana">Número de registres per plana</param>
        /// <returns>Llistat de (numRegsPlana + 1) registres, per tal de que 
        /// es pugui activar o no el botó de següent (si es retorna 
        /// (numRegsPlana + 1) registres, es pot mostrar el botó de següent).</returns>
        IList<ItemLlistaOfertes> ConsultarOfertesPaginat(ParamsSearchOfertes parametres,
            int numPlana, int numRegsPlana);

        /// <summary>
        /// Guarda la oferta a la base de dades. 
        /// Si no existeix es crea un nou registre, i si existeix 
        /// s'actualitza el registre segons les dades subministrades.
        /// </summary>
        /// <param name="empresa">Dades de la oferta</param>
        /// <returns>PK de la nova oferta guardada o actualitzada</returns>
        /// <exception cref="GaudeixBusinessException">En cas d'error durant l'execució del mètode</exception>
        int GuardarOfertaFromWCF(DetallOfertaWcf oferta);

        /// <summary>
        /// Obté el detall d'una oferta a partir de la seva clau primaria
        /// </summary>
        /// <param name="idEmpresa">Clau primària de la oferta</param>
        /// <returns>Detall de la oferta</returns>
        /// <exception cref="GaudeixBusinessException">En cas d'error durant l'execució del mètode</exception>
        DetallOfertaWcf GetDetallOfertaForWCF(int idOferta, bool recuperarImatge);

        /// <summary>
        /// Obté el detall d'una oferta per a mostrar-ho a la web
        /// </summary>
        /// <param name="idOferta">Clau primària de la oferta</param>
        /// <returns>Detall de la oferta</returns>
        DetallOfertaModel GetDetallOfertaForWeb(int idOferta);

        /// <summary>
        /// Permet esborrar una o més ofertes del sistema
        /// </summary>
        /// <param name="idOfertes">Codis d'oferta a esborrar</param>
        /// <exception cref="GaudeixBusinessException">En cas d'error durant l'execució del mètode</exception>
        void EsborrarOfertes(int[] idOfertes);

        /// <summary>
        /// Actualitza les estadístiques d'una oferta determinada
        /// </summary>
        /// <param name="idOferta">Codi de la oferta</param>
        /// <returns></returns>
        void ActualitzarEstadistiques(int idOferta);

        /// <summary>
        /// Incrementa en una unitat el número de visites d'una oferta
        /// </summary>
        /// <param name="idOferta">Codi de la oferta</param>
        void AfegirVisita(int idOferta);
        #endregion
    }
}
