﻿using System;
using System.Collections.Generic;
using Gaudeix.Common.Infrastructure.Manager;
using Gaudeix.Common.Models.Users;
using Gaudeix.Common.Models.Account;
using Gaudeix.Common.Models.Profile;

namespace Gaudeix.Common.Interfaces.Manager
{
    /// <summary>
    /// Interficie de declaracio d'operacions sobre usuaris per a la capa de negoci
    /// </summary>
    public interface IUsuariManager : IBaseManager
    {
        #region Mètodes a implementar
        /// <summary>
        /// Valida si les credencials d'un usuari són vàlides per a poder accedir al sistema
        /// </summary>
        /// <param name="username">Nom d'usuari</param>
        /// <param name="password">Paraula clau</param>
        /// <returns>True si les credencials són vàlides, False en cas contrari</returns>
        /// <exception cref="GaudeixBusinessException">En cas d'error durant l'execució del mètode</exception>
        bool ValidateUser(String username, String password);

        /// <summary>
        /// Retorna un llistat dels usuaris que compleixen els paràmetres de cerca.
        /// </summary>
        /// <returns>Llista d'usuaris que compleixen amb els paràmetres de cerca</returns>
        /// <exception cref="GaudeixBusinessException">En cas d'error durant l'execució del mètode</exception>
        IList<ItemLlistaUsuaris> ConsultarUsuaris(ParamsSearchUsuaris parametres);

        /// <summary>
        /// Guarda l'usuari a la base de dades. 
        /// Si no existeix es crea un nou registre, i si existeix 
        /// s'actualitza el registre segons les dades subministrades.
        /// </summary>
        /// <param name="usuari">Dades de l'usuari</param>
        /// <param name="updatePassword">Indica si s'ha d'actualitzar l'usuari</param>
        /// <returns>PK del nou usuari creat o actualitzat</returns>
        /// <exception cref="GaudeixBusinessException">En cas d'error durant l'execució del mètode</exception>
        int GuardarUsuari(DetallUsuari usuari, bool updatePassword);

        /// <summary>
        /// Obté el detall d'un usuari a partir de la seva clau primaria
        /// </summary>
        /// <param name="idEmpresa">Clau primària de l'usuari</param>
        /// <returns>Detall de l'usuari</returns>
        DetallUsuari GetDetallUsuariById(int idUsuari);

        /// <summary>
        /// Obté el detall d'un usuari a partir del seu codi d'usuari
        /// </summary>
        /// <param name="userName">Nom d'usuari</param>
        /// <returns>Detall de l'usuari</returns>
        DetallUsuari GetDetallUsuariByUserName(string userName);

        /// <summary>
        /// Permet esborrar una o més usuaris del sistema
        /// </summary>
        /// <param name="idOfertes">Codis d'usuari a esborrar</param>
        /// <exception cref="GaudeixBusinessException">En cas d'error durant l'execució del mètode</exception>
        void EsborrarUsuaris(int[] idUsuaris);

        /// <summary>
        /// Indica si existeix un usuari a partir del seu username
        /// </summary>
        /// <param name="userName">Codi d'usuari a validar</param>
        /// <returns>true si existeix, false en cas contrari</returns>
        bool ExisteixUsuariByUserName(string userName);

        /// <summary>
        /// Indica si existeix un usuari a partir del seu nif
        /// </summary>
        /// <param name="nif">NIF a validar</param>
        /// <returns>true si existeix, false en cas contrari</returns>
        bool ExisteixUsuariByNIF(string nif);

        /// <summary>
        /// Canvia la contrasenya d'un usuari especificant la contrasenya antiga
        /// </summary>
        /// <param name="userName">Nom d'usuari</param>
        /// <param name="oldPassword">Contrasenya antiga</param>
        /// <param name="newPassword">Nova contrasenya</param>
        /// <param name="newPasswordConfirm">Confirmació de la nova contrasenya</param>
        /// <returns></returns>
        bool CanviarContrasenya(string userName, string oldPassword, string newPassword, string newPasswordConfirm);

        /// <summary>
        /// Permet modificar una contrasenya fent servir la resposta a la pregunta de seguretat
        /// </summary>
        /// <param name="userName">Codi d'usuari a modificar</param>
        /// <param name="respostaSeguretat">Resposta a la pregunta de seguretat</param>
        /// <param name="newPassword">Nova contrasenya</param>
        /// <param name="newPasswordConfirm">Confirmació de la nova contrasenya</param>
        /// <returns>Indicador de si s'ha pogut modificar la contrasenya</returns>
        bool CanviarContrasenyaPreguntaSeguretat(string userName, string respostaSeguretat, 
            string newPassword, string newPasswordConfirm);

        /// <summary>
        /// Indica si una oferta ha estat comprada per un usuari
        /// </summary>
        /// <param name="idOferta">Codi de la oferta a validar</param>
        /// <param name="idUsuari">Codi d'usuari a validar</param>
        /// <returns>true si l'oferta ha estat comprada per l'usuari, false en cas contrari</returns>
        bool OfertaCompradaPerUsuari(int idOferta, int idUsuari);

        /// <summary>
        /// Obté un llistat de tots els correus electrònics dels usuaris registrats
        /// </summary>
        /// <returns>Llista de correus electrònics</returns>
        IList<String> GetAllEmailsFromUsers();

        /// <summary>
        /// Obté totes les categories del sistema, indicant si estan associades o no a l'usuari passat
        /// </summary>
        /// <param name="idUsuari">Codi d'usuari pel filtre. Si es passa un '0', es retornen totes les categories.</param>
        /// <returns>LLista de categories</returns>
        IList<ItemLlistaCategories> GetCategoriesUsuari(int idUsuari);
        #endregion
    }
}
