﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.IO;

namespace Gaudeix.Common.Utils
{
    /// <summary>
    /// Classe d'utilitat per a encriptar i desencriptar cadenes amb AES - Rijndael
    /// </summary>
    public static class CryptoUtils
    {
        #region Mètodes públics
        /// <summary>
        /// Encripta una contrasenya, utilitzant aquesta constrasenya com a clau d'encriptació. 
        /// Com a vector d'inicialització es fa servir el username, el qual serà únic per a totes 
        /// les contrasenyes. D'aquesta forma, una mateixa contrasenya tindrà valors encriptats diferents.
        /// </summary>
        /// <param name="username">Nom d'usuari. Es fa servir per a generar el vector d'inicialització.</param>
        /// <param name="password">Contrasenya a encriptar</param>
        /// <returns>Contrasenya encriptada</returns>
        public static string EncryptPassword(string username, string password)
        {
            byte[] IV = GetIV(username);
            byte[] key = GetKey(password);
            return encryptString(password, key, IV);
        }

        /// <summary>
        /// Desencripta una paraula clau. 
        /// Per a desencriptar-la farà falta saber el nom d'usuari associat 
        /// (utilitzat com a vector d'inicialització), així com la contrasenya 
        /// en text clar (és la clau de desencriptació).
        /// </summary>
        /// <param name="username">Nom d'usuari. Es fa servir per a generar el vector d'inicialització.</param>
        /// <param name="cryptPassword">Contrasenya encriptada</param>
        /// <param name="plainPassword">Contrasenya en text clar</param>
        /// <returns></returns>
        public static string DecryptPassword(string username, string cryptPassword, string plainPassword)
        {
            try
            {
                byte[] IV = GetIV(username);
                byte[] key = GetKey(plainPassword);
                return decryptString(cryptPassword, key, IV);
            }
            catch (Exception)
            {
                return null;
            }
        }
        #endregion

        #region Mètodes privats
        /// <summary>
        /// Obté el vector d'inicialització a partir d'un string. Si la mida del 
        /// paràmetre és inferior a 16, s'afegiran caràcters al final.
        /// </summary>
        /// <param name="IV">String per a generar el Vector d'inicialitació</param>
        /// <returns>Vector d'inicialitació</returns>
        private static byte[] GetIV(string IV)
        {
            // La longitud mínima ha de ser de 16 bytes.
            // Si la mida és menor, afegirem caracters al final de l'array
            while (IV.Length < 16)
            {
                IV += IV;
            }
            // Retornem només els 16 primers bytes en majúscules
            return UTF8Encoding.UTF8.GetBytes(IV.ToUpper().Substring(0, 16));
        }

        /// <summary>
        /// Obté la clau d'encriptació / desencriptació a partir d'un string. Si la mida del 
        /// paràmetre és inferior a 16, s'afegiran caràcters al final.
        /// </summary>
        /// <param name="key">String per a generar la clau</param>
        /// <returns>Clau d'encriptació / desencriptació</returns>
        private static byte[] GetKey(string key)
        {
            // La longitud mínima ha de ser de 16 bytes.
            // Si la mida és menor, afegirem caracters al final de l'array
            while (key.Length < 16)
            {
                key += "#";
            }
            return UTF8Encoding.UTF8.GetBytes(key);
        }


        /// <summary>
        /// Encripta una cadena de text amb l'agoritme AES - Rijndael
        /// </summary>
        /// <param name="plainMessage">Missatge a encriptar</param>
        /// <param name="Key">Clau d'encriptació</param>
        /// <param name="IV">Vector d'inicialització</param>
        /// <returns>Valor encriptat</returns>
        private static string encryptString(String plainMessage, byte[] Key, byte[] IV)
        {
            // Crear una instancia de l'algorisme de Rijndael  
            Rijndael RijndaelAlg = Rijndael.Create();

            MemoryStream memoryStream = new MemoryStream();
            // Crear un fluxe de xifratge 
            CryptoStream cryptoStream = new CryptoStream(memoryStream,
                                                         RijndaelAlg.CreateEncryptor(Key, IV),
                                                         CryptoStreamMode.Write);

            // Obtenir la representació en bytes de la informació a xifrar
            byte[] plainMessageBytes = UTF8Encoding.UTF8.GetBytes(plainMessage);
            // Encriptar les dades enviant-los al flux de xifrat
            cryptoStream.Write(plainMessageBytes, 0, plainMessageBytes.Length);
            cryptoStream.FlushFinalBlock();
            // Obtenim les dades encriptades com una matriu de bytes
            byte[] cipherMessageBytes = memoryStream.ToArray();
            // Cerrar los flujos utilizados  
            memoryStream.Close();
            cryptoStream.Close();
            // Retornem la cadena xifrada con un string en Base64
            return Convert.ToBase64String(cipherMessageBytes);
        }

        /// <summary>
        /// Desencripta una cadena de text amb l'algorisme de Rijndael (AES)
        /// </summary>
        /// <param name="encryptedMessage">Missatge xifrat</param>
        /// <param name="Key">Clau de desencriptació</param>
        /// <param name="IV">Vector d'inicialització</param>
        /// <returns>Missatge desencriptat</returns>
        private static string decryptString(String encryptedMessage, byte[] Key, byte[] IV)
        {
            // Obtenir la representació en bytes de la informació a desencriptar
            byte[] cipherTextBytes = Convert.FromBase64String(encryptedMessage);
            byte[] plainTextBytes = new byte[cipherTextBytes.Length];
            // Crear una instancia de l'algorisme de Rijndael  
            Rijndael RijndaelAlg = Rijndael.Create();

            MemoryStream memoryStream = new MemoryStream(cipherTextBytes);

            CryptoStream cryptoStream = new CryptoStream(memoryStream,
                                                         RijndaelAlg.CreateDecryptor(Key, IV),
                                                         CryptoStreamMode.Read);
            int decryptedByteCount = cryptoStream.Read(plainTextBytes, 0, plainTextBytes.Length);

            memoryStream.Close();
            cryptoStream.Close();

            return Encoding.UTF8.GetString(plainTextBytes, 0, decryptedByteCount);
        }
        #endregion
    }
}
