﻿using System;
using System.Text.RegularExpressions;

namespace Gaudeix.Common.Utils
{
    /// <summary>
    /// Classe que conté mètodes de validació de dades
    /// </summary>
    public static class ValidationUtils
    {
        #region Mètodes públics
        /// <summary>
        /// Comprova si una cadena de text té el format d'una adreça de correu electrònic
        /// </summary>
        /// <param name="mail">Cadena a validar</param>
        /// <returns>True si la cadena es correspon a una adreça de correu, fals en cas contrari</returns>
        public static bool IsEmail(String mail)
        {
            Regex re = new Regex(@"^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$", RegexOptions.IgnoreCase);
            return re.IsMatch(mail);
        }

        /// <summary>
        /// Funcio que valida si una cadena de text és un CIF vàlid
        /// </summary>
        /// <param name="cif">CIF a validar, sense separadors (punts ni guions)</param>
        /// <returns>True si correspon a un CIF vàlid, False si no ho és</returns>
        public static bool IsCifValid(String cif)
        {
            //Valida el cif actual
            string[] lletresCodi = { "J", "A", "B", "C", "D", "E", "F", "G", "H", "I" };

            string LletraInicial = cif[0].ToString();
            string DigitControl = cif[cif.Length - 1].ToString();
            string n = cif.ToString().Substring(1, cif.Length - 2);
            int sumaParells = 0;
            int sumaSenars = 0;
            int sumaTotal = 0;
            int i = 0;
            bool retVal = false;

            // Recorregut per tots els dígits del número
            for (i = 0; i < n.Length; i++)
            {
                int aux;
                Int32.TryParse(n[i].ToString(), out aux);

                if ((i + 1) % 2 == 0)
                {
                    // Si és una posició parell, es sumen els dígits
                    sumaParells += aux;
                }
                else
                {
                    // Si es una posició senar, es multipliquen els dígits per 2
                    aux = aux * 2;
                    // S'afegeixen els dígits a la suma global
                    sumaSenars += SumaDigitos(aux);
                }
            }

            // Sumem el resultat dels números parells i senars
            sumaTotal += sumaParells + sumaSenars;

            // Obtenim el dígit de les unitats
            Int32 unitats = sumaTotal % 10;

            // Si les unitats són diferents de 0, es resten de 10
            if (unitats != 0) unitats = 10 - unitats;

            switch (LletraInicial)
            {
                // Només valors numèrics
                case "A":
                case "B":
                case "E":
                case "H":
                    retVal = DigitControl == unitats.ToString();
                    break;

                // Només lletres
                case "K":
                case "P":
                case "Q":
                case "S":
                    retVal = DigitControl == lletresCodi[unitats];
                    break;

                default:
                    retVal = (DigitControl == unitats.ToString()) || (DigitControl == lletresCodi[unitats]);
                    break;
            }

            return retVal;
        }

        /// <summary>Calcula si un nif és vàlid</summary>
        /// <param name="nif">NIF a processar, en format 00000000X</param>
        /// <returns>Booleà que indica si és vàlid o no</returns>
        public static bool IsNifValid(string nif)
        {
            string lletres = "TRWAGMYFPDXBNJZSQVHLCKE";

            if ((string.IsNullOrEmpty(nif)) || (nif.Length != 9))
            {
                // Si és buit o amb longitud diferent a 9, és incorrecte
                return false;
            }
            int dni;
            if (!int.TryParse(nif.Substring(0, 8), out dni))
            {
                // Les vuit primeres posicions no són numèriques
                return false;
            }

            // Calculem la lletra
            string lletraNIF = lletres[dni % 23].ToString();
            return lletraNIF.Equals(nif.Substring(8, 1), StringComparison.InvariantCultureIgnoreCase);
        }

        /// <summary>
        /// Valida si una tarjeta de crèdit és vàlida. 
        /// Per a ser-ho, ha de tenir 16 dígits, han de ser tots numèrics 
        /// i els darrers tres dígits han de ser iguals al codi de validació
        /// </summary>
        /// <param name="numTarjaCredit">Número de la tarja de crèdit</param>
        /// <param name="codiValidacio">Codi de validació de la tarja</param>
        /// <returns>True si és vàlida, false en cas contrari</returns>
        public static bool IsTarjaValida(string numTarjaCredit, string codiValidacio)
        {
            // Numero de tarja obligatori
            if (String.IsNullOrEmpty(numTarjaCredit))
            {
                return false;
            }

            // Codi de validació obligatori
            if (String.IsNullOrEmpty(codiValidacio))
            {
                return false;
            }

            // Longitud tarja de credit: 16
            if (numTarjaCredit.Length != 16)
            {
                return false;
            }

            // Longitud de codi de validació: 3
            if (codiValidacio.Length != 3)
            {
                return false;
            }

            // Numero tarja ha de ser numèric
            if (ParseUtils.ParseDecimal(numTarjaCredit) == 0)
            {
                return false;
            }

            // Codi de validacio ha de ser numèric
            if (ParseUtils.ParseInt32(codiValidacio) == 0)
            {
                return false;
            }

            // El codi de validació és correcte si es correspon 
            // als darrers tres digits de la tarja de crèdit
            return numTarjaCredit.EndsWith(codiValidacio);
        }

        #endregion

        #region Mètodes privats
        
        /// <summary>
        /// Realitza la suma de tots els dígits que composen el número passat
        /// </summary>
        /// <param name="digitos">Dígits a sumar</param>
        /// <returns>Suma de tots els dígits</returns>
        private static Int32 SumaDigitos(Int32 digitos)
        {
            string sNumero = digitos.ToString();
            Int32 suma = 0;

            for (Int32 i = 0; i < sNumero.Length; i++)
            {
                Int32 aux;
                Int32.TryParse(sNumero[i].ToString(), out aux);
                suma += aux;
            }
            return suma;
        }


        #endregion

    }
}
