﻿using System;
using Castle.Core.Logging;
using Gaudeix.Common.Exceptions;
using Gaudeix.Common.Infrastructure.Dao;
using Gaudeix.Common.Infrastructure.Entity;
using Gaudeix.Common.Resources;
using Gaudeix.Data.Infrastructure.Factory;
using NHibernate;

namespace Gaudeix.Data.Infrastructure.Dao
{
    /// <summary>
    /// Classe base de la que deriven tots els DAO's
    /// </summary>
    /// <typeparam name="TEntity">Entitat de NHibernate gestionada pel DAO</typeparam>
    /// <typeparam name="TKey">Tipus de la clau primària de la entitat</typeparam>
    public abstract class BaseDao<TEntity, TKey> : IBaseDao<TEntity, TKey>
        where TEntity : BaseEntity
        where TKey : IComparable
    {
        #region Propietats de la classe

        // Controlador de LOG. S'injecta a través del contenidor Castle.
        public ILogger logger { get; set; }
        #endregion

        #region IBaseDao<TModel,TKey> Members

        public TEntity GetById(TKey id)
        {
            try
            {
                return Session.Get<TEntity>(id);
            }
            catch (Exception ex)
            {
                logger.Error("BaseDao.GetById", ex);
                throw new GaudeixDaoException(CommonErrors.ErrorBD, ex);
            }
        }

        public TKey Save(TEntity entity)
        {
            try
            {
                object ent = Session.Save((object)entity);
                return (TKey)ent;
            }
            catch (Exception ex)
            {
                logger.Error("BaseDao.Save", ex);
                throw new GaudeixDaoException(CommonErrors.ErrorBD, ex);
            }
        }

        public TKey SaveOrUpdate(TEntity entity)
        {
            try
            {
                if (entity.isNewEntity)
                {
                    return this.Save(entity);
                }
                else
                {
                    return this.Update(entity);
                }
            }
            catch (Exception ex)
            {
                logger.Error("BaseDao.SaveOrUpdate", ex);
                throw new GaudeixDaoException(CommonErrors.ErrorBD, ex);
            }
        }

        public TKey Update(TEntity entity)
        {
            try
            {
                // S'ha de treure la entitat de la sessió amb el mètode "Evict", ja que si no no ho actualitza
                Session.Evict(entity);

                Session.Update(entity);
                return (TKey)entity.PrimaryKey;
            }
            catch (Exception ex)
            {
                logger.Error("BaseDao.Update", ex);
                throw new GaudeixDaoException(CommonErrors.ErrorBD, ex);
            }
        }

        public void Delete(TEntity entity)
        {
            try
            {
                // S'ha de treure la entitat de la sessió amb el mètode "Evict", ja que si no no ho actualitza
                Session.Evict(entity);

                Session.Delete((object)entity);
            }
            catch (PropertyValueException ex)
            {
                logger.Error("BaseDao.Delete", ex);
                throw new GaudeixDaoException(CommonErrors.ErrorDependenciesRegistre, ex);
            }
            catch (Exception ex)
            {
                logger.Error("BaseDao.Delete", ex);
                throw new GaudeixDaoException(CommonErrors.ErrorBD, ex);
            }
        }

        #endregion

        #region Mètodes interns o protegits
        /// <summary>
        /// Retorna la sessió actual de NHibernate
        /// </summary>
        protected ISession Session
        {
            get
            {
                try
                {
                    return NHSessionFactory.GetCurrentSession();
                }
                catch (Exception ex)
                {
                    logger.Error("BaseDao.Session", ex);
                    throw new GaudeixDaoException(CommonErrors.ErrorBD, ex);
                }
            }
        }
        #endregion
    }
}
