﻿using System;
using NHibernate;
using NHibernate.Cfg;
using NHibernate.Context;

namespace Gaudeix.Data.Infrastructure.Factory
{
    /// <summary>
    /// Classe estàtica encarregada de servir de factoria d'instàncies de les classes DAO.
    /// La classe està segellada (sealed) per a evitar possibles modificacions en el seu comportament.
    /// </summary>
    public sealed class NHSessionFactory
    {
        #region Variables i propietats privades de la classe
        /// <summary>
        /// La instància de ISessionFactory és estàtica degut a que la creació 
        /// d'una nova instància és un procés molt pesat.
        /// Al crear-la a través d'un patró Lazy la instància es crearà, de forma 
        /// sincronitzada entre tots els processos que hi accedeixin, la primera vegada que es faci servir.
        /// </summary>
        private static readonly Lazy<ISessionFactory> factory = new Lazy<ISessionFactory>(() =>
        {
            Configuration configuration = new Configuration();
            configuration.Configure();
            return configuration.BuildSessionFactory();
        }, System.Threading.LazyThreadSafetyMode.ExecutionAndPublication);

        /// <summary>
        /// Accessor de lectura del ISessionFactory
        /// </summary>
        private static ISessionFactory SessionFactory
        {
            get
            {
                return factory.Value;
            }
        }
        #endregion

        #region Mètodes públics de la classe
        /// <summary>
        /// Obté la sessió de NHibernate associada a la request actual.
        /// </summary>
        /// <returns>Sessió de NHibernate actual</returns>
        public static ISession GetCurrentSession()
        {
            if (!CurrentSessionContext.HasBind(SessionFactory))
            {
                CurrentSessionContext.Bind(SessionFactory.OpenSession());
            }
            //ISession sess = SessionFactory.GetCurrentSession();
            //sess.FlushMode = FlushMode.Commit;
            ////sess.CacheMode = CacheMode.Get;
            //return sess;
            return SessionFactory.GetCurrentSession();
        }

        /// <summary>
        /// Realitza la lògica necessaria per a alliberar la sessió actual.
        /// </summary>
        public static void DisposeSession()
        {
            ISession session = GetCurrentSession();
            if (session.IsOpen)
            {
                FlushMode mode = session.FlushMode;
               
                session.Flush();
                session.Clear();
                session.Close();
            }
            session.Dispose();
        }

        /// <summary>
        /// Inicia una nova transacció a base de dades
        /// </summary>
        public static void BeginTransaction()
        {
            GetCurrentSession().BeginTransaction();
        }

        /// <summary>
        /// Confirma a la base de dades els canvis realitzats per la transacció actual
        /// </summary>
        public static void CommitTransaction()
        {
            ISession session = GetCurrentSession();
            // Validem que la sessió estigui oberta, i que la transacció estigui activa
            if ((session.IsOpen) & (session.Transaction.IsActive))
            {
                session.Transaction.Commit();
            }
        }

        /// <summary>
        /// Reverteix els canvis efectuats per la transacció actual
        /// </summary>
        public static void RollbackTransaction()
        {
            ISession session = GetCurrentSession();
            if ((session.IsOpen) && (session.Transaction.IsActive))
            {
                // Revertim els canvis a la base de dades
                session.Transaction.Rollback();
            }
        }
        #endregion
    }
}


