﻿using System;
using System.Web;
using System.Web.Routing;
using Gaudeix.Data.Infrastructure.Factory;
using System.Diagnostics;
using Castle.Core.Logging;
using NHibernate;

namespace Gaudeix.Data.Infrastructure.HttpModules
{
    /// <summary>
    /// Classe que implementa un mòdul IHttpModule, per tal de gestionar el patrò 
    /// Transaction-Per-Request a les sessions de NHibernate, i d'aquesta forma 
    /// poder centralitzar la gestió de les transaccions.
    /// </summary>
    public sealed class NHibernateHttpSessionModule : IHttpModule
    {

        #region IHttpModule Members

        /// <summary>
        ///  Allibera els recursos utilitzats pel mòdul
        /// </summary>
        public void Dispose() { }

        /// <summary>
        /// Inicialitza el mòdul i el prepara per a gestionar les peticions.
        /// </summary>
        /// <param name="context">Proporciona accés als mètodes, propietats i events que son comuns a tots els objectes de l'aplicació ASP.NET</param>
        public void Init(HttpApplication context)
        {
            // Al iniciar la request, s'obre una nova transacció
            context.BeginRequest += new EventHandler(BeginTransaction);

            // Al finalitzar la request, es fa un commit de la transacció, i es tanca la sessió
            context.EndRequest += new EventHandler(CommitAndCloseSession);
        }

        #endregion

        #region Controladors d'events del mòdul
        /// <summary>
        /// Event que s'executa al iniciar-se una petició HTTP i que serveix per aplicar el patró Trasaction-Per-Request. 
        /// Si aquesta petició sol·licita una acció d'un controlador, llavors obre una transacció.
        /// Obre una transacció a l'inici d'una petició HTTP.
        /// </summary>
        /// <param name="sender">Origen de l'event</param>
        /// <param name="e">Arguments de l'event</param>
        private void BeginTransaction(object sender, EventArgs e)
        {
            HttpContext context = ((HttpApplication)sender).Context;
            RouteData routeData = RouteTable.Routes.GetRouteData(new HttpContextWrapper(context));
            if ((routeData != null) && (routeData.Values["controller"] != null))
            {
                // Si no s'especifica un nom de controlador, no cal obrir transacció.
                // Per exemple, es pot demanar el favicon, un fitxer d'script o una imatge
                NHSessionFactory.BeginTransaction();
            }
        }

        /// <summary>
        /// Confirma els canvis efectuats per la transacció actual (en el cas de que s'hagi obert 
        /// a l'inici de la request, i posteriorment tanca la sessió de NHibernate.
        /// </summary>
        /// <param name="sender">Origen de l'event</param>
        /// <param name="e">Arguments de l'event</param>
        private void CommitAndCloseSession(object sender, EventArgs e)
        {
            NHSessionFactory.CommitTransaction();
            NHSessionFactory.DisposeSession();
        }
        #endregion
    }
}
