﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Mail;
using System.Text;
using Gaudeix.Common.Constants;
using Gaudeix.Common.Domain.Entities;
using Gaudeix.Common.Exceptions;
using Gaudeix.Common.Interfaces.Dao;
using Gaudeix.Common.Interfaces.Manager;
using Gaudeix.Common.Models.Browse;
using Gaudeix.Common.Models.Common;
using Gaudeix.Common.Resources;
using Gaudeix.Common.Utils;
using Gaudeix.Manager.Infrastructure.Base;

namespace Gaudeix.Business.Manager
{
    public class CuaCorreuManager : BaseManager, ICuaCorreuManager
    {
        #region Mètodes privats

        /// <summary>
        /// Obté el client per enviament per SMTP, inicialitzant els valors des de la base de dades
        /// </summary>
        /// <returns>Client SMTP inicialitzat</returns>
        private SmtpClient GetClientSmtp()
        {
            try
            {
                IParametreDao dao = this.GetDao<IParametreDao>();
                SmtpClient client = new SmtpClient();
                string User = dao.GetParamValueByCodi(CommonConstants.PARAM_SMTP_USER);
                string Password = dao.GetParamValueByCodi(CommonConstants.PARAM_SMTP_PASSWORD);
                client.Credentials = new NetworkCredential(User, Password);
                client.Port = ParseUtils.ParseInt32(dao.GetParamValueByCodi(CommonConstants.PARAM_SMTP_PORT));
                client.Host = dao.GetParamValueByCodi(CommonConstants.PARAM_SMTP_SERVER);
                client.EnableSsl = ParseUtils.ParseBool(dao.GetParamValueByCodi(CommonConstants.PARAM_SMTP_SSL));
                client.Timeout = ParseUtils.ParseInt32(dao.GetParamValueByCodi(CommonConstants.PARAM_SMTP_TIMEOUT));
                return client;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("Error al carregar la configuració del servei de correu electrònic", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        /// <summary>
        /// Recupera l'adreça del remitent des de la base de dades
        /// </summary>
        /// <returns>Adreça del remitent</returns>
        private MailAddress GetSenderMailAddress()
        {
            try
            {
                IParametreDao dao = this.GetDao<IParametreDao>();
                string address = dao.GetParamValueByCodi(CommonConstants.PARAM_SMTP_SENDER_ADDRESS);
                string name = dao.GetParamValueByCodi(CommonConstants.PARAM_SMTP_SENDER_NAME);
                return new MailAddress(address, name, Encoding.UTF8);
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("Error al carregar l'adreça del remitent", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        /// <summary>
        /// Envia les ofertes passades com a paràmetre als usuaris que tinguin seleccionada la categoria indicada
        /// </summary>
        /// <param name="idCategoria">Categoria d'enviament</param>
        /// <param name="ofertes">Llistat d'ofertes</param>
        private void SendOfertesUsuarisByPreferencies(int idCategoria, IList<Oferta> ofertes)
        {
            if ((ofertes ==null) || (ofertes.Count==0)){
                return;
            }
            // Preparem el llistat d'ofertes
            StringBuilder llistaOfertes = new StringBuilder();
            foreach (Oferta oferta in ofertes)
            {
                llistaOfertes.Append(String.Format("{0} - {1}{2}", oferta.Titol,
                    FormatUtils.FormatImport(oferta.PreuVenda),"<br/>"));
                llistaOfertes.Append("Ofertat per " + oferta.Empresa.Nom  + "<br/>");
                llistaOfertes.Append("<br/>");
            }

            // Recuperem plantilla correu
            IParametreDao paramDao = this.GetDao<IParametreDao>();
            string mailPlantilla = paramDao.GetParamValueByCodi(CommonConstants.PARAM_MAIL_OFERTES);

            // Afegim camps comuns
            string nomCategoria = this.GetDao<ICategoriaDao>().GetById(idCategoria).Descripcio;
            mailPlantilla = mailPlantilla.Replace(CommonConstants.FIELD_NOM_CATEGORIA, nomCategoria);
            mailPlantilla =mailPlantilla.Replace(CommonConstants.FIELD_LLISTA_OFERTES,llistaOfertes.ToString());

            // Recuperem els usuaris que tenen activada aquesta categoria
            IUsuariDao userDao = GetDao<IUsuariDao>();
            IList<Usuari> usuaris = userDao.GetMailUsersByIdCategoria(idCategoria);
            string mail;
            string titol = String.Format("Noves ofertes en la categoria '{0}'",nomCategoria);
            foreach (Usuari usuari in usuaris)
            {
                mail = mailPlantilla;
                mail = mail.Replace(CommonConstants.FIELD_NOM_RECEPTOR,
                    FormatUtils.FormatName(usuari.Nom, usuari.Cognom1,usuari.Cognom2));
                this.EnqueueMail(usuari.Email,titol,mail);
            }
        }
        #endregion

        #region IMailManager Members

        public void EnqueueMail(string destinatari, string titol, string message)
        {
            try
            {
                // Creem el nou correu electrònic
                CuaCorreu correu = new CuaCorreu();
                correu.Destinatari = destinatari;
                correu.Enviat = false;
                correu.Text = message;
                correu.Titol = titol;

                // Guardem el nou correu a la BD
                ICuaCorreuDao dao = this.GetDao<ICuaCorreuDao>();
                dao.Save(correu);

            }
            catch (GaudeixDaoException ex)
            {
                // Control d'excepcions del DAO
                logger.Error("MailManager.EnqueueMail", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public void EnqueueMailRecommend(RecommendPopUpModel model, string linkOferta)
        {
            try
            {
                // Recuperem plantilla correu
                IParametreDao paramDao = this.GetDao<IParametreDao>();
                string mail = paramDao.GetParamValueByCodi(CommonConstants.PARAM_MAIL_RECOMANACIO);

                // Substituim els valors parametritzats
                mail = mail.Replace(CommonConstants.FIELD_MISSATGE_OPCIONAL, model.Missatge);
                mail = mail.Replace(CommonConstants.FIELD_NOM_RECEPTOR, model.NomAmic);
                mail = mail.Replace(CommonConstants.FIELD_NOM_RECOMANAT, model.NomPropi);
                mail = mail.Replace(CommonConstants.FIELD_LINK_OFERTA, linkOferta);

                this.EnqueueMail(model.MailAmic, model.Titol, mail);
            }
            catch (GaudeixDaoException ex)
            {
                // Control d'excepcions del DAO
                logger.Error("MailManager.EnqueueMail", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public void ProcessMailQueue()
        {
            try
            {
                // Inicialitzem el DAO
                ICuaCorreuDao dao = this.GetDao<ICuaCorreuDao>();
                // Inicialitzem client SMTP
                SmtpClient client = GetClientSmtp();
                // Obtenir llistat dels correus pendents
                IList<CuaCorreu> correus = dao.GetCorreusPendents();
                MailAddress remitent = GetSenderMailAddress();
                MailAddress destinatari;
                MailMessage missatge;
                bool enviat;
                foreach (CuaCorreu correu in correus)
                {
                    // Adreça del destinatari
                    destinatari = new MailAddress(correu.Destinatari);

                    // Cos del missatge
                    missatge = new MailMessage(remitent, destinatari);
                    missatge.Body = correu.Text;
                    missatge.BodyEncoding = Encoding.UTF8;
                    missatge.IsBodyHtml = true;
                    missatge.Subject = correu.Titol;
                    missatge.SubjectEncoding = Encoding.UTF8;
                    try
                    {
                        // Enviem el missatge
                        client.Send(missatge);
                        enviat = true;
                    }
                    catch (Exception ex)
                    {
                        // Ha ocorregut un error. Guardem traça d'error al fitxer de log.
                        logger.Error("Error al enviar el correu", ex);
                        enviat = false;
                    }
                    if (enviat)
                    {
                        // Missatge enviat, actualitzem estat de la cua a la BD.
                        //correu.Enviat = true;
                        //dao.SaveOrUpdate(correu);
                        dao.Delete(correu);
                    }

                }
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("Error al processar la llista de correus pendents", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public void ProcessMailOfertes()
        {
            try
            {
                ICategoriaDao dao = this.GetDao<ICategoriaDao>();
                IList<ItemIdDescripcio> categories = dao.GetCategories();
                IOfertaDao ofertaDao = this.GetDao<IOfertaDao>();
                IList<Oferta> ofertes = null;
                foreach (ItemIdDescripcio categoria in categories)
                {
                    ofertes = ofertaDao.GetTopOfertes(categoria.Id);
                    if (ofertes.Count > 0)
                    {
                        this.SendOfertesUsuarisByPreferencies(categoria.Id, ofertes);
                    }
                }
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("Error al processar la llista de correus pendents", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public void EnqueueMailCompraUsuari(int idCompra, string linkOferta)
        {
            // Recuperem plantilla correu
            IParametreDao paramDao = this.GetDao<IParametreDao>();
            string mail = paramDao.GetParamValueByCodi(CommonConstants.PARAM_MAIL_COMPRA_USER);
            CompraUsuari compra = this.GetDao<ICompraUsuariDao>().GetById(idCompra);

            // Substituim els valors parametritzats
            mail = mail.Replace(CommonConstants.FIELD_NOM_RECEPTOR,
                FormatUtils.FormatNameSurname(compra.Usuari.Nom, compra.Usuari.Cognom1, compra.Usuari.Cognom2));
            mail = mail.Replace(CommonConstants.FIELD_TITOL_OFERTA, compra.Oferta.Titol);
            mail = mail.Replace(CommonConstants.FIELD_LINK_OFERTA, linkOferta);
            mail = mail.Replace(CommonConstants.FIELD_ID_COMPRA, idCompra.ToString());
            mail = mail.Replace(CommonConstants.FIELD_DATA_COMPRA, FormatUtils.FormatData(compra.DataCompra));
            Usuari usuari = this.GetDao<IUsuariDao>().GetById(compra.Usuari.Id);
            this.EnqueueMail(usuari.Email, CommonFields.SubjectConfirmacioCompra, mail);
        }

        public void EnqueueMailCompraEmpresa(int idCompra, string linkOferta)
        {
            // Recuperem plantilla correu
            IParametreDao paramDao = this.GetDao<IParametreDao>();
            string mail = paramDao.GetParamValueByCodi(CommonConstants.PARAM_MAIL_COMPRA_EMPRESA);
            CompraUsuari compra = this.GetDao<ICompraUsuariDao>().GetById(idCompra);

            // Substituim els valors parametritzats
            mail = mail.Replace(CommonConstants.FIELD_NOM_CONTACTES, compra.Oferta.Empresa.Contactes);
            mail = mail.Replace(CommonConstants.FIELD_TITOL_OFERTA, compra.Oferta.Titol);
            mail = mail.Replace(CommonConstants.FIELD_LINK_OFERTA, linkOferta);
            mail = mail.Replace(CommonConstants.FIELD_NOM_COMPRADOR,
                FormatUtils.FormatNameSurname(compra.Usuari.Nom, compra.Usuari.Cognom1, compra.Usuari.Cognom2));
            mail = mail.Replace(CommonConstants.FIELD_DOCUMENT_COMPRADOR, compra.Usuari.Nif);
            mail = mail.Replace(CommonConstants.FIELD_DATA_COMPRA, FormatUtils.FormatData(compra.DataCompra));
            this.EnqueueMail(compra.Oferta.Empresa.Email, CommonFields.SubjectNovaCompra, mail);
        }

        public void EnqueueMailBenvinguda(int idUsuari)
        {
            // Recuperem plantilla correu
            IParametreDao paramDao = this.GetDao<IParametreDao>();
            string mail = paramDao.GetParamValueByCodi(CommonConstants.PARAM_MAIL_BENVINGUDA);
            Usuari usuari = this.GetDao<IUsuariDao>().GetById(idUsuari);

            // Substituim els valors parametritzats
            mail = mail.Replace(CommonConstants.FIELD_NOM_RECEPTOR,
                FormatUtils.FormatNameSurname(usuari.Nom, usuari.Cognom1, usuari.Cognom2));
            this.EnqueueMail(usuari.Email, CommonFields.SubjectBenvinguda, mail);
        }

        #endregion
    }


}
