﻿using System;
using System.Collections.Generic;
using System.Text;
using Gaudeix.Common.Domain.Entities;
using Gaudeix.Common.Exceptions;
using Gaudeix.Common.Interfaces.Dao;
using Gaudeix.Common.Interfaces.Manager;
using Gaudeix.Common.Models.Empresa;
using Gaudeix.Common.Resources;
using Gaudeix.Common.Utils;
using Gaudeix.Data.Infrastructure.Factory;
using Gaudeix.Manager.Infrastructure.Base;

namespace Gaudeix.Business.Manager
{
    public class EmpresaManager : BaseManager, IEmpresaManager
    {
        #region IEmpresaManager Members

        public int GuardarEmpresa(DetallEmpresa empresa)
        {
            StringBuilder errors = new StringBuilder();

            // Validacions d'integritat
            // Validar nom obligatori
            if (String.IsNullOrEmpty(empresa.Nom))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Nom));
            }

            // Validar CIF obligatori
            if (String.IsNullOrEmpty(empresa.Cif))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Cif));
            }
            else if (!ValidationUtils.IsCifValid(empresa.Cif))
            {
                // Validar CIF
                errors.AppendLine(String.Format(CommonErrors.CifInvalid, CommonFields.Cif));
            }

            if (String.IsNullOrEmpty(empresa.Email))
            {
                errors.AppendLine(string.Format(CommonErrors.CampObligatori, CommonFields.EMail));
            }
            else if (!ValidationUtils.IsEmail(empresa.Email))
            {
                errors.AppendLine(string.Format(CommonErrors.FormatIncorrecte, CommonFields.EMail));
            }

            if (empresa.IdPoblacio <= 0)
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Poblacio));
            }

            if (errors.Length > 0)
            {
                throw new GaudeixBusinessException(errors.ToString());
            }

            try
            {
                Empresa emp = new Empresa();
                emp.Id = empresa.Id;
                emp.Nom = empresa.Nom;
                emp.Cif = empresa.Cif;
                emp.Adreca = empresa.Adreca;
                emp.Email = empresa.Email;
                if (empresa.IdPoblacio > 0)
                {
                    emp.Poblacio = new Poblacio(empresa.IdPoblacio);
                }
                else
                {
                    emp.Poblacio = null;
                }
                emp.Contactes = empresa.Contactes;

                // Cridem al DAO per guardar les dades
                IEmpresaDao dao = this.GetDao<IEmpresaDao>();
                return dao.SaveOrUpdate(emp);
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("EmpresaManager.GuardarEmpresa", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public IList<ItemLlistaEmpreses> ConsultarEntitats(ParamsSearchEmpreses parametres)
        {
            // Validar que s'hagi informat algun paràmetre
            if (parametres.NumParametresInformats == 0)
            {
                throw new GaudeixBusinessException(CommonErrors.NoParametresCerca);
            }

            try
            {
                return this.GetDao<IEmpresaDao>().ConsultarEntitats(parametres);
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("EmpresaManager.ConsultarEntitats", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public DetallEmpresa GetDetallEmpresa(int idEmpresa)
        {
            try
            {
                IEmpresaDao dao = this.GetDao<IEmpresaDao>();
                // Recuperem les dades de l'empresa
                DetallEmpresa empresa = dao.GetDetallEmpresa(idEmpresa);
                // Recuperem les ofertes de l'empresa
                empresa.Ofertes = dao.GetOfertesEmpresa(idEmpresa);
                return empresa;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("EmpresaManager.GetDetallEmpresa", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public IList<string> GetAllEmailsFromEmpreses()
        {
            try{
                return this.GetDao<IEmpresaDao>().GetAllEmailsFromEmpreses();
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("EmpresaManager.GetAllEmailsFromEmpreses", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public void EsborrarEmpreses(int[] idEmpreses)
        {
            try{
                IEmpresaDao dao = this.GetDao<IEmpresaDao>();
                foreach(int idEmpresa in idEmpreses){
                    dao.Delete(new Empresa(idEmpresa));
                }
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("EmpresaManager.EsborrarEmpreses", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        #endregion
    }
}
