﻿using System;
using System.Collections.Generic;
using System.Text;
using Gaudeix.Common.Domain.Entities;
using Gaudeix.Common.Exceptions;
using Gaudeix.Common.Interfaces.Dao;
using Gaudeix.Common.Interfaces.Manager;
using Gaudeix.Common.Models.Browse;
using Gaudeix.Common.Models.Oferta;
using Gaudeix.Common.Resources;
using Gaudeix.Manager.Infrastructure.Base;

namespace Gaudeix.Manager.Messages
{
    public class OfertaManager : BaseManager, IOfertaManager
    {
        #region IOfertaManager Members

        public IList<ItemLlistaOfertes> ConsultarOfertes(ParamsSearchOfertes parametres)
        {
            return this.ConsultarOfertesPaginat(parametres, -1, -1);
        }

        public IList<ItemLlistaOfertes> ConsultarOfertesPaginat(ParamsSearchOfertes parametres,
             int numPlana, int numRegsPlana)
        {
            // Validar que s'hagi informat algun paràmetre
            if (parametres.NumParametresInformats <= 0)
            {
                throw new GaudeixBusinessException(CommonErrors.NoParametresCerca);
            }
            try
            {
                return this.GetDao<IOfertaDao>().ConsultarOfertesPaginat(parametres,numPlana,numRegsPlana);
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("EmpresaManager.ConsultarEntitats", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public int GuardarOfertaFromWCF(DetallOfertaWcf oferta)
        {
            StringBuilder errors = new StringBuilder();
            // Validacions d'integritat
            if (oferta.IdEmpresa <= 0)
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Empresa));
            }
            if (oferta.DataInici == new DateTime(0))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.DataInici));
            }
            if (oferta.DataFi == new DateTime(0))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.DataFi));
            }

            if (oferta.PreuBase <= 0)
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.PreuBase));
            }
            if (oferta.PreuVenda <= 0)
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.PreuVenda));
            }

            if (String.IsNullOrEmpty(oferta.Titol))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Titol));
            }
            if (oferta.IdPoblacio <= 0)
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Poblacio));
            }
            if (oferta.IdCategoria <= 0)
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Categoria));
            }
            if (String.IsNullOrEmpty(oferta.Descripcio))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Descripcio));
            }
            if (String.IsNullOrEmpty(oferta.DadesResum))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.DadesResum));
            }
            if (String.IsNullOrEmpty(oferta.DadesAddicionals))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.DadesAddicionals));
            }
            if (String.IsNullOrEmpty(oferta.DadesLocalitzacio))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.DadesLocalitzacio));
            }

            if (errors.Length > 0)
            {
                throw new GaudeixBusinessException(errors.ToString());
            }

            try
            {
                Oferta of = new Oferta();
                of.Id = oferta.IdOferta;
                of.Empresa = new Empresa(oferta.IdEmpresa);
                of.Poblacio = new Poblacio(oferta.IdPoblacio);
                of.DataInici = oferta.DataInici;
                of.DataFi = oferta.DataFi;
                of.PreuBase = oferta.PreuBase;
                of.PreuVenda = oferta.PreuVenda;
                of.Titol = oferta.Titol;
                of.Descripcio = oferta.Descripcio;
                of.NotaMitja = oferta.NotaMitja;
                of.DadesResum = oferta.DadesResum;
                of.DadesAddicionals = oferta.DadesAddicionals;
                of.DadesLocalitzacio = oferta.DadesLocalitzacio;
                of.NumVisites = oferta.NumVisites;
                of.NumCompres = oferta.NumCompres;
                of.Categoria = new Categoria(oferta.IdCategoria);

                // Cridem al DAO per guardar les dades
                IOfertaDao dao = this.GetDao<IOfertaDao>();
                int idOferta = dao.SaveOrUpdate(of);
                if (oferta.Imatge != null)
                {
                    IFotoOfertaManager fotoManager = this.GetManager<IFotoOfertaManager>();
                    fotoManager.GuardarFotoOferta(idOferta, oferta.Imatge);
                }
                return idOferta;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("EmpresaManager.GuardarEmpresa", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public DetallOfertaWcf GetDetallOfertaForWCF(int idOferta, bool recuperarImatge)
        {
            try
            {
                IOfertaDao dao = this.GetDao<IOfertaDao>();
                DetallOfertaWcf result = dao.GetDetallOfertaForWCF(idOferta);
                if (recuperarImatge)
                {
                    IFotoOfertaManager fotoManager = this.GetManager<IFotoOfertaManager>();
                    result.Imatge = fotoManager.GetImageFromOferta(idOferta);
                }
                return result;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("OfertaManager.GetDetallOferta", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public void EsborrarOfertes(int[] idOfertes)
        {
            try
            {
                IOfertaDao dao = this.GetDao<IOfertaDao>();
                foreach (int idOferta in idOfertes)
                {
                    dao.Delete(new Oferta(idOferta));
                }
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("OfertaManager.EsborrarOfertes", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public DetallOfertaModel GetDetallOfertaForWeb(int idOferta)
        {
            try
            {
                IOfertaDao dao = this.GetDao<IOfertaDao>();
                // Obtenim les dades generals de l'oferta
                DetallOfertaModel result = dao.GetDetallOfertaForWeb(idOferta);
                // Obtenim el llistat de comentaris
                result.Comentaris = this.GetDao<IComentariOfertaDao>().GetComentarisOferta(idOferta);
                return result;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("OfertaManager.GetDetallOfertaForWeb", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public void ActualitzarEstadistiques(int idOferta)
        {
            try
            {
                IOfertaDao dao = this.GetDao<IOfertaDao>();
                dao.ActualitzarEstadistiques(idOferta);
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("OfertaManager.actualitzarEstadistiques", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public void AfegirVisita(int idOferta)
        {
            try
            {
                IOfertaDao dao = this.GetDao<IOfertaDao>();
                dao.AfegirVisita(idOferta);
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("OfertaManager.actualitzarEstadistiques", ex);
                throw new GaudeixBusinessException(ex);
            }
        }
        #endregion
    }
}
