﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Gaudeix.Common.Domain.Entities;
using Gaudeix.Common.Exceptions;
using Gaudeix.Common.Interfaces.Dao;
using Gaudeix.Common.Interfaces.Manager;
using Gaudeix.Common.Models.Users;
using Gaudeix.Common.Resources;
using Gaudeix.Common.Utils;
using Gaudeix.Data.Infrastructure.Factory;
using Gaudeix.Manager.Infrastructure.Base;
using Gaudeix.Common.Models.Profile;
using Gaudeix.Common.Models.Common;

namespace Gaudeix.Manager.Manager
{
    public class UsuariManager : BaseManager, IUsuariManager
    {
        #region IUsuariManager Members

        public bool ExisteixUsuariByUserName(string userName)
        {
            try
            {
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                Usuari user = dao.GetByUsername(userName);
                return (user != null);
            }
            catch (GaudeixDaoException ex)
            {
                // Control d'excepcions del DAO
                logger.ErrorFormat(ex, "IUsuariManager.ExisteixUsuariByUserName:{0}", userName);
                throw new GaudeixBusinessException(ex);
            }
        }

        public bool ExisteixUsuariByNIF(string nif)
        {
            try
            {
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                Usuari user = dao.GetByNif(nif);
                return (user != null);
            }
            catch (GaudeixDaoException ex)
            {
                // Control d'excepcions del DAO
                logger.ErrorFormat(ex, "IUsuariManager.ExisteixUsuariByNif:{0}", nif);
                throw new GaudeixBusinessException(ex);
            }
        }

        public bool ValidateUser(string username, string password)
        {
            try
            {
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                Usuari user = dao.GetByUsername(username);

                if (null == user)
                {
                    // No s'ha trobat l'usuari a la base de dades
                    logger.InfoFormat("No s'ha trobat a l'usuari {0} a la base de dades", username);
                    return false;
                }
                else
                {
                    string passDecrypt = CryptoUtils.DecryptPassword(user.LoginUsuari, user.LoginPassword, password);
                    if (String.IsNullOrEmpty(CryptoUtils.DecryptPassword(user.LoginUsuari, user.LoginPassword, password)))
                    {
                        // No s'ha desencriptat el password: El password és incorrecte
                        return false;
                    }
                    else
                    {
                        return (password.Equals(passDecrypt));
                    }
                }
            }
            catch (GaudeixDaoException ex)
            {
                // Control d'excepcions del DAO
                logger.ErrorFormat(ex, "IUsuariManager.ValidateUser", username);
                throw new GaudeixBusinessException(ex);
            }
            catch (Exception ex)
            {
                // Control d'altres exceptions
                logger.ErrorFormat(ex, "IUsuariManager.ValidateUser", username);
                throw new GaudeixBusinessException(CommonErrors.ErrorBusiness, ex);
            }
        }

        public IList<ItemLlistaUsuaris> ConsultarUsuaris(ParamsSearchUsuaris parametres)
        {
            // Validar que s'hagi informat algun paràmetre
            if (parametres.NumParametresInformats == 0)
            {
                throw new GaudeixBusinessException(CommonErrors.NoParametresCerca);
            }
            try
            {
                return this.GetDao<IUsuariDao>().ConsultarOfertes(parametres);
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("UsuariManager.ConsultarUsuaris", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public int GuardarUsuari(DetallUsuari usuari, bool updatePassword)
        {
            // Validacions d'integritat
            string errors = this.ValidarDetallUsuari(usuari, updatePassword);

            if (errors.Length > 0)
            {
                throw new GaudeixBusinessException(errors);
            }

            try
            {
                Usuari user;
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                bool usuariExistent = (usuari.IdUsuari > 0);
                if (usuariExistent)
                {
                    user = dao.GetById(usuari.IdUsuari);
                }
                else
                {
                    user = new Usuari();
                }
                user.Poblacio = new Poblacio(usuari.IdPoblacio);
                user.Nom = usuari.Nom;
                user.Cognom1 = usuari.Cognom1;
                user.Cognom2 = usuari.Cognom2;
                user.Nif = usuari.Nif;
                user.Email = usuari.Email;
                user.Adreca = usuari.Adreca;
                user.CodiPostal = usuari.CodiPostal;
                user.DataNaixement = usuari.DataNaixement;
                user.LoginUsuari = usuari.LoginUsuari;

                if (updatePassword)
                {
                    user.LoginPassword = CryptoUtils.EncryptPassword(usuari.LoginUsuari, usuari.LoginPassword);
                }
                user.PreguntaClau = usuari.PreguntaClau;
                user.RespostaClau = usuari.RespostaClau;
                user.MailOfertes = usuari.MailOfertes;
                int idUser = dao.SaveOrUpdate(user);

                ICategoriaUsuariDao catUserDao = this.GetDao<ICategoriaUsuariDao>();

                // Actualitzem les categories de l'usuari
                IEnumerable<ItemLlistaCategories> categoriesSeleccionades = usuari.LlistaCategories.Where(c => c.CategoriaActiva);
                foreach (CategoriaUsuari cat in user.CategoriesUsuari)
                {
                    if (categoriesSeleccionades.Where(c => c.IdCategoria == cat.Id).Count() == 0)
                    {
                        // S'ha d'esborrar la categoria
                        catUserDao.Delete(cat);
                    }
                }
                foreach (ItemLlistaCategories item in categoriesSeleccionades)
                {
                    if (user.CategoriesUsuari.Where(c => c.Id == item.IdCategoria).Count() == 0)
                    {
                        // S'ha d'afegir la categoria
                        CategoriaUsuari categoria = new CategoriaUsuari();
                        categoria.Usuari = user;
                        categoria.Categoria = new Categoria(item.IdCategoria);
                        catUserDao.Save(categoria);
                    }
                }
                if (!usuariExistent)
                {
                    // Enviar correu de benvinguda
                    this.GetManager<ICuaCorreuManager>().EnqueueMailBenvinguda(idUser);
                }

                return idUser;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("UsuariManager.GuardarUsuari", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public DetallUsuari GetDetallUsuariByUserName(string userName)
        {
            try
            {
                DetallUsuari detall;
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                Usuari user = dao.GetByUsername(userName);
                if (user == null)
                {
                    throw new GaudeixBusinessException(CommonErrors.UsuariInexistent);
                }
                detall = this.GetDetallUsuariById(user.Id);
                return detall;
            }
            catch (GaudeixDaoException ex)
            {
                // Control d'excepcions del DAO
                logger.ErrorFormat(ex, "IUsuariManager.GetDetallUsuariByUserName:{0}", userName);
                throw new GaudeixBusinessException(ex);
            }
        }

        public DetallUsuari GetDetallUsuariById(int idUsuari)
        {
            DetallUsuari detall;
            try
            {
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                detall = dao.GetDetallUsuari(idUsuari);
                ICompraUsuariDao compresDao = this.GetDao<ICompraUsuariDao>();
                detall.LlistaCompres = compresDao.GetCompresUsuari(detall.IdUsuari);
                // Carregar categories usuari
                detall.LlistaCategories = this.GetCategoriesUsuari(idUsuari);
                return detall;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("UsuariManager.GetDetallUsuariById", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public void EsborrarUsuaris(int[] idUsuaris)
        {
            try
            {
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                foreach (int idUsuari in idUsuaris)
                {
                    dao.Delete(new Usuari(idUsuari));
                }
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("UsuariManager.EsborrarUsuaris", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public bool CanviarContrasenya(string username, string oldPassword, string newPassword, string newPasswordConfirm)
        {
            try
            {
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                Usuari usuari = dao.GetByUsername(username);

                if (String.IsNullOrEmpty(CryptoUtils.DecryptPassword(username, usuari.LoginPassword, oldPassword)))
                {
                    logger.InfoFormat("Intent de password erroni de l'usuari {0}", username);
                    throw new GaudeixBusinessException(CommonErrors.ErrorOldPassword);
                }
                usuari.LoginPassword = CryptoUtils.EncryptPassword(username, newPassword);
                dao.Update(usuari);
                //dao.Save(usuari);
                return true;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("UsuariManager.CanviarContrasenya", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public bool OfertaCompradaPerUsuari(int idOferta, int idUsuari)
        {
            try
            {
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                Usuari usuari = dao.GetById(idUsuari);
                int num = usuari.CompresUsuari.Count(c => (c.Usuari.Id == idUsuari) && (c.Oferta.Id == idOferta));
                return num > 0;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("UsuariManager.OfertaCompradaPerUsuari", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public bool CanviarContrasenyaPreguntaSeguretat(string userName, string respostaSeguretat,
            string newPassword, string newPasswordConfirm)
        {
            try
            {
                StringBuilder errors = new StringBuilder();
                if (string.IsNullOrEmpty(userName))
                {
                    errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.LoginUsername));
                }
                if (string.IsNullOrEmpty(respostaSeguretat))
                {
                    errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.PreguntaSeguretat));
                }
                if (string.IsNullOrEmpty(newPassword))
                {
                    errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.LoginPassword));
                }
                else if (!newPassword.Equals(newPasswordConfirm))
                {
                    errors.AppendLine(CommonErrors.ParaulesClauDiferents);
                }
                if (errors.Length > 0)
                {
                    throw new GaudeixBusinessException(errors.ToString());
                }
                IUsuariDao dao = this.GetDao<IUsuariDao>();
                Usuari usuari = dao.GetByUsername(userName);
                if (!respostaSeguretat.Equals(usuari.RespostaClau))
                {
                    logger.InfoFormat("Resposta de seguretat incorrecta per l'usuari {0}", userName);
                    throw new GaudeixBusinessException(CommonErrors.ErrorRespostaSeguretat);
                }
                // Si arribem fins aquí, es pot canviar el password
                usuari.LoginPassword = CryptoUtils.EncryptPassword(userName, newPassword);
                dao.Update(usuari);
                return true;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("UsuariManager.CanviarContrasenyaPreguntaSeguretat", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public IList<string> GetAllEmailsFromUsers()
        {
            try
            {
                return this.GetDao<IUsuariDao>().GetAllEmailsFromUsers();
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("IUsuariManager.GetAllEmailsFromUsers", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        public IList<ItemLlistaCategories> GetCategoriesUsuari(int idUsuari)
        {
            try
            {
                IList<ItemLlistaCategories> resultat = new List<ItemLlistaCategories>();
                ICategoriaDao categoriaDao = this.GetDao<ICategoriaDao>();
                IList<ItemIdDescripcio> categories = categoriaDao.GetCategories();

                ICategoriaUsuariDao catUserDao = this.GetDao<ICategoriaUsuariDao>();
                IList<ItemIdDescripcio> catUser = catUserDao.GetCategoriesUsuari(idUsuari);
                bool assignat;
                foreach (ItemIdDescripcio item in categories)
                {
                    assignat = (catUser.Where(c => c.Id == item.Id).Count() > 0);
                    resultat.Add(new ItemLlistaCategories(assignat, item.Id, item.Descripcio));
                }
                return resultat;
            }
            catch (GaudeixDaoException ex)
            {
                logger.Error("UsuariManager.GetCategoriesUsuari", ex);
                throw new GaudeixBusinessException(ex);
            }
        }

        #endregion
        #region Mètodes privats
        /// <summary>
        /// Valida que el detall d'un usuari sigui correcte abans de guardar-ho
        /// </summary>
        /// <param name="usuari">Dades a validar</param>
        /// <param name="updatePassword">Indica si s'ha d'actualitzar el password</param>
        /// <returns>true si és correcte, false en cas contrari</returns>
        private string ValidarDetallUsuari(DetallUsuari usuari, bool updatePassword)
        {
            StringBuilder errors = new StringBuilder();

            // Nom obligatori
            if (String.IsNullOrEmpty(usuari.Nom))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Nom));
            }

            // Cognom1 obligatori
            if (String.IsNullOrEmpty(usuari.Cognom1))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Cognom1));
            }

            // NIF Obligatori i ben format
            if (String.IsNullOrEmpty(usuari.Nif))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Nif));
            }
            else if (!ValidationUtils.IsNifValid(usuari.Nif))
            {
                errors.AppendLine(String.Format(CommonErrors.NifInvalid, CommonFields.Nif));
            }

            // Correu electrònic obligatori i ben format
            if (String.IsNullOrEmpty(usuari.Email))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.EMail));
            }
            else if (!ValidationUtils.IsEmail(usuari.Email))
            {
                errors.AppendLine(String.Format(CommonErrors.EmailIncorrecte, CommonFields.EMail));
            }

            // Adreça obligatoria
            if (String.IsNullOrEmpty(usuari.Adreca))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Adreca));
            }

            // Poblacio obligatoria
            if (usuari.IdPoblacio <= 0)
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.Poblacio));
            }

            // Codi Postal obligatori i numèric
            if (String.IsNullOrEmpty(usuari.CodiPostal))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.CodiPostal));
            }
            else if (ParseUtils.ParseInt32(usuari.CodiPostal) <= 0)
            {
                errors.AppendLine(String.Format(CommonErrors.CampNumeric, CommonFields.CodiPostal));
            }

            // Data de naixement obligatoria i usuari major d'edat
            if (usuari.DataNaixement == new DateTime(0))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.DataNaixement));
            }
            else
            {
                double edadDbl = DateTime.Now.Subtract(usuari.DataNaixement).TotalDays / 365.25;
                int edad = (int)edadDbl;
                if (edad < 18)
                {
                    errors.AppendLine(CommonErrors.UsuariMenorEdat);
                }
            }

            // Login usuari obligatori
            if (String.IsNullOrEmpty(usuari.LoginUsuari))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.LoginUsername));
            }

            if (updatePassword)
            {
                // Password obligatori
                if ((usuari.IdUsuari <= 0) && (string.IsNullOrEmpty(usuari.LoginPassword)))
                {
                    errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.LoginPassword));
                }

                // Els passwords han de coincidir
                if (!usuari.LoginPassword.Equals(usuari.LoginPasswordConfirm))
                {
                    errors.AppendLine(CommonErrors.ParaulesClauDiferents);
                }
            }

            // Pregunta clau obligatoria
            if (String.IsNullOrEmpty(usuari.PreguntaClau))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.PreguntaSeguretat));
            }

            // Resposta pregunta clau obligatoria
            if (String.IsNullOrEmpty(usuari.RespostaClau))
            {
                errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.RespostaSeguretat));
            }
            return errors.ToString();
        }
        #endregion

    }
}
