﻿using System.Text.RegularExpressions;
using System.Web.Mvc;
using System.Web.Security;
using Gaudeix.Common.Exceptions;
using Gaudeix.Common.Interfaces.Manager;
using Gaudeix.Common.Models.Account;
using Gaudeix.Common.Models.Users;
using Gaudeix.Common.Resources;
using Gaudeix.Manager.Infrastructure.Factory;
using Gaudeix.Web.Constants;
using Gaudeix.Web.Infrastructure.Controllers;
using Gaudeix.Web.Resources;
using Gaudeix.Web.Security;
using Gaudeix.Common.Constants;


namespace Gaudeix.Web.Controllers
{
    public class AccountController : BaseController
    {

        // GET: /Account/LogOn
        public ActionResult LogOn()
        {
            return View();
        }

        // POST: /Account/LogOn
        [HttpPost]
        public ActionResult LogOn(LogOnModel model, string returnUrl)
        {
            Logger.Info("INICI - POST: /Account/LogOn/");
            try
            {
                if (ModelState.IsValid)
                {
                    if (Membership.ValidateUser(model.UserName, model.Password))
                    {
                        FormsAuthentication.SetAuthCookie(model.UserName, model.RememberMe);
                        if (Url.IsLocalUrl(returnUrl) && returnUrl.Length > 1 && returnUrl.StartsWith("/")
                            && !returnUrl.StartsWith("//") && !returnUrl.StartsWith("/\\"))
                        {
                            Logger.Info("INICI - POST: /Account/LogOn/ -> Validació correcta, redirecció a plana sol.licitada");
                            return Redirect(returnUrl);
                        }
                        else
                        {
                            Logger.Info("INICI - POST: /Account/LogOn/ -> Validació correcta, redirecció a plana d'inici");
                            return RedirectToAction(AppConstants.DEFAULT_ACTION, AppConstants.DEFAULT_CONTROLLER);
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("", GlobalMessages.ErrorUsuariPassword);
                    }
                }
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - POST: /Account/LogOn/ -> Validació erronia");
            // Si arribem aquí, alguna cosa ha fallat, així que recarreguem la plana
            return View(model);
        }

        // GET: /Account/LogOff
        public ActionResult LogOff()
        {
            Logger.Info("INICI - GET: /Account/LogOff/");
            FormsAuthentication.SignOut();
            Logger.Info("INICI - GET: /Account/LogOff/");
            return RedirectToAction(AppConstants.DEFAULT_ACTION, AppConstants.DEFAULT_CONTROLLER);
        }


        // GET: /Account/Register
        public ActionResult Register()
        {
            Logger.Info("INICI - GET: /Account/Register/");

            RegisterModel model = new RegisterModel();
            try
            {
                // Recuperem la llista de provincies
                model.LlistaProvincies = this.GetManager<ITaulesBaseManager>().GetProvincies();
                // Carregar categories
                model.LlistaCategories = this.GetManager<IUsuariManager>().GetCategoriesUsuari(0);
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - GET: /Account/Register/");
            return View(model);
        }

        // POST: /Account/Register
        [HttpPost]
        public ActionResult Register(RegisterModel model)
        {
            Logger.Info("INICI - POST: /Account/Register/");
            try
            {
                if (ModelState.IsValid)
                {
                    // Intentem registrar a l'usuari
                    MembershipCreateStatus createStatus;
                    ((GaudeixMembershipProvider)Membership.Provider).CreateUser(model, out createStatus);
                    if (createStatus == MembershipCreateStatus.Success)
                    {
                        // L'usuari s'ha autenticat correctament.
                        // Guardem una cookie d'autenticació, i retornem a l'index
                        FormsAuthentication.SetAuthCookie(model.UsuariLogin, false);
                        Logger.Info("FI - POST: /Account/Register/ -> Registre correcte, redirecció a plana per defecte");
                        return RedirectToAction(AppConstants.DEFAULT_ACTION, AppConstants.DEFAULT_CONTROLLER);
                    }
                    else
                    {
                        ModelState.AddModelError("", ErrorCodeToString(createStatus));
                    }
                }


                // Si hem arribat aquí es que ha fallat alguna cosa, així que retornem la vista
                ITaulesBaseManager manager = GetManager<ITaulesBaseManager>();
                // Recuperem la llista de provincies
                model.LlistaProvincies = manager.GetProvincies();
                if (model.idProvincia > 0)
                {
                    model.LlistaPoblacions = manager.GetPoblacionsByIdProvincia(model.idProvincia);
                }
            }
            catch (GaudeixMembershipException ex)
            {
                string[] errors = Regex.Split(ex.Message, "\r\n");
                foreach (string error in errors)
                {
                    ModelState.AddModelError("", error);
                }

            }


            Logger.Info("FI - POST: /Account/Register/");
            return View(model);
        }


        // GET: /Account/RememberPasswdPopUp
        public PartialViewResult RememberPasswdPopUp(string id)
        {
            Logger.Info("INICI - GET: /Account/RememberPasswdPopUp/");
            RememberPasswordModel model = new RememberPasswordModel();
            try
            {
                IUsuariManager manager = this.GetManager<IUsuariManager>();
                DetallUsuari user = manager.GetDetallUsuariByUserName(id);
                model.Username = user.LoginUsuari;
                model.PreguntaSeguretat = user.PreguntaClau;
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - GET: /Account/RememberPasswdPopUp/");
            return PartialView(model);
        }

        // POST: /Account/RememberPasswdPopUp
        [HttpPost]
        public JsonResult RememberPasswdPopUp(RememberPasswordModel model)
        {
            Logger.Info("INICI - POST: /Account/RememberPasswdPopUp/");
            string error = string.Empty;
            bool changeOk = false;
            try
            {
                IUsuariManager manager = this.GetManager<IUsuariManager>();
                changeOk = manager.CanviarContrasenyaPreguntaSeguretat(model.Username, model.RespostaSeguretat,
                    model.NouPassword, model.NouPasswordConfirm);
            }
            catch (GaudeixBusinessException ex)
            {
                changeOk = false;
                error = ex.Message.Replace("\r\n", "<br/>");
            }
            JsonResult result = new JsonResult();
            result.Data = new { returnValue = changeOk, errorMessage = error };
            Logger.Info("FI - POST: /Account/RememberPasswdPopUp/");
            return result;
        }

        #region Mètodes privats
        /// <summary>
        /// Converteix un codi d'error de creació d'usuari a un missatge que es pot presentar a l'usuari
        /// </summary>
        /// <param name="createStatus">Codi d'error produït</param>
        /// <returns>Missatge entenible per l'usuari</returns>
        private static string ErrorCodeToString(MembershipCreateStatus createStatus)
        {
            switch (createStatus)
            {
                case MembershipCreateStatus.DuplicateProviderUserKey:
                    return CommonErrors.ErrorNifDuplicat;
                case MembershipCreateStatus.DuplicateUserName:
                    return CommonErrors.ErrorUserNameDuplicat;
                case MembershipCreateStatus.ProviderError:
                    return CommonErrors.ErrorProvider;
                default:
                    return CommonErrors.ErrorProvider;
            }
        }


        #endregion
    }
}
