﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Web.Mvc;
using Gaudeix.Common.Exceptions;
using Gaudeix.Common.Interfaces.Manager;
using Gaudeix.Common.Models.Browse;
using Gaudeix.Common.Models.Common;
using Gaudeix.Common.Models.Oferta;
using Gaudeix.Common.Utils;
using Gaudeix.Manager.Infrastructure.Factory;
using Gaudeix.Web.Infrastructure.Controllers;
using Gaudeix.Common.Constants;

namespace Gaudeix.Web.Controllers
{
    /// <summary>
    /// Controlador d'operacions sobre les ofertes del sistema
    /// </summary>
    public class BrowseController : BaseController
    {
        #region Mètodes privats
        /// <summary>
        /// Inicialitza el model de cerca d'ofertes
        /// </summary>
        /// <param name="model">Model a actualitzar</param>
        /// <returns>Model actualitzat</returns>
        private SearchModel initBrowseDealsModel(SearchModel model)
        {
            // Recuperem la llista de provincies
            ITaulesBaseManager manager = this.GetManager<ITaulesBaseManager>();
            model.LlistaProvincies = manager.GetProvincies();
            int idProvincia = ParseUtils.ParseInt32(model.idProvincia);
            if (idProvincia > 0)
            {
                model.LlistaPoblacions = manager.GetPoblacionsByIdProvincia(idProvincia);
            }
            else
            {
                model.LlistaPoblacions = new List<ItemIdDescripcio>();
            }
            model.LlistaCategories = manager.GetCategories();
            // Afegir llista ordenacions
            model.LlistaOrdenacions = CommonConstants.GetLlistaOrdre();
            return model;
        }

        /// <summary>
        /// Obté els paràmetres de cerca d'ofertes a partir del model
        /// </summary>
        /// <param name="model">Model a carregar</param>
        /// <returns>Paràmetres de cerca</returns>
        private ParamsSearchOfertes GetParamsSearchOfertesFromModel(SearchModel model)
        {
            ParamsSearchOfertes parametres = new ParamsSearchOfertes();
            parametres.idProvincia = ParseUtils.ParseInt32(model.idProvincia);
            parametres.idPoblacio = ParseUtils.ParseInt32(model.idPoblacio);
            parametres.idCategories = new int[] { ParseUtils.ParseInt32(model.idCategoria) };
            if (!String.IsNullOrEmpty(model.preuDesDe))
            {
                parametres.PreuVendaInit = ParseUtils.ParseDecimal(model.preuDesDe);
            }
            if (!String.IsNullOrEmpty(model.preuFinsA))
            {
                parametres.PreuVendaFi = ParseUtils.ParseDecimal(model.preuFinsA);
            }
            parametres.Titol = model.TextOferta;
            parametres.IdOrdenacio = model.idOrdenacio.HasValue ? model.idOrdenacio.Value : 0;
            return parametres;
        }

        /// <summary>
        /// Obté el número de registres per plana cofnigurat a la BBDD
        /// </summary>
        /// <returns>Número de registres per plana, o CommonConstants.PAGINACIO_MIN_PAGINACIO en cas d'error</returns>
        private int GetNumRegistresPlana()
        {
            IParametreManager parametres = this.GetManager<IParametreManager>();
            int numRegistres = ParseUtils.ParseInt32(parametres.GetParamValueByCodi(CommonConstants.PARAM_NUM_REGISTRES_PLANA));
            if (numRegistres < CommonConstants.PAGINACIO_MIN_PAGINACIO)
            {
                numRegistres = CommonConstants.PAGINACIO_MIN_PAGINACIO;
            }
            return numRegistres;
        }

        #endregion

        #region Actions del controller
        // GET: /Browse/Deals
        public ActionResult Deals()
        {
            Logger.Info("INICI - GET: /Browse/Deals");
            SearchModel model = new SearchModel();
            try
            {
                int numRegistresPlana = this.GetNumRegistresPlana();
                model = initBrowseDealsModel(model);
                IOfertaManager manager = this.GetManager<IOfertaManager>();
                model.LlistaResultats = manager.ConsultarOfertesPaginat(this.GetParamsSearchOfertesFromModel(model), model.NumPlanaPaginacio, numRegistresPlana);
                // Inicialitzem número de plana
                model.NumPlanaPaginacio = 0;
                model.ShowAnterior = (model.NumPlanaPaginacio > 0);
                model.ShowSeguent = (model.LlistaResultats.Count() > numRegistresPlana);
                if (model.LlistaResultats.Count > numRegistresPlana)
                {
                    // Hi ha un registre més dels que toquen, usat per a mostrar el botó de següent
                    model.LlistaResultats.RemoveAt(model.LlistaResultats.Count - 1);
                }
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - GET: /Browse/Deals");

            return View(model);
        }

        // POST: /Browse/Deals/
        [HttpPost]
        public ActionResult Deals(SearchModel model)
        {
            Logger.Info("INICI - POST: /Browse/Deals");
            try
            {
                int numRegistresPlana = this.GetNumRegistresPlana();
                IOfertaManager manager = this.GetManager<IOfertaManager>();
                model = initBrowseDealsModel(model);
                int i = model.NumPlanaPaginacio;
                model.LlistaResultats = manager.ConsultarOfertesPaginat(this.GetParamsSearchOfertesFromModel(model), model.NumPlanaPaginacio, numRegistresPlana);
                model.ShowAnterior = (model.NumPlanaPaginacio > 0);
                model.ShowSeguent = (model.LlistaResultats.Count() > numRegistresPlana);
                if (model.LlistaResultats.Count > numRegistresPlana)
                {
                    // Hi ha un registre més dels que toquen, usat per a mostrar el botó de següent
                    model.LlistaResultats.RemoveAt(model.LlistaResultats.Count - 1);
                }
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - GET: /Browse/Deals");
            return View(model);
        }

        // GET: /Browse/Detail
        public ActionResult Detail(int id)
        {
            Logger.Info("INICI - GET: /Browse/Detail/" + id.ToString());
            DetallOfertaModel model = new DetallOfertaModel();
            try
            {
                IOfertaManager manager = this.GetManager<IOfertaManager>();
                // Recuperem la informació de la BBDD
                model = manager.GetDetallOfertaForWeb(id);
                // Actualitzem el número de visites
                manager.AfegirVisita(id);
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - GET: /Browse/Detail/" + id.ToString());
            return View(model);
        }

        // GET: /Browse/BuyPopUp
        [Authorize]
        public PartialViewResult BuyPopUp(int id)
        {
            Logger.Info("INICI - GET: /Browse/BuyPopUp/" + id.ToString());
            BuyPopUpModel model = new BuyPopUpModel();
            try
            {
                IOfertaManager manager = this.GetManager<IOfertaManager>();
                DetallOfertaModel oferta = manager.GetDetallOfertaForWeb(id);
                model.IdOferta = id;
                model.PreuVenda = oferta.PreuVenda;
                model.Titol = oferta.Titol;
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - GET: /Browse/BuyPopUp/" + id.ToString());
            return PartialView(model);
        }

        // POST: /Browse/BuyPopUp
        [HttpPost]
        [Authorize]
        public JsonResult BuyPopUp(BuyPopUpModel model)
        {
            Logger.Info("INICI - POST: /Browse/BuyPopUp/");
            bool changeOk = false;
            string error = String.Empty;
            try
            {
                ICompraUsuariManager manager = this.GetManager<ICompraUsuariManager>();
                string url = String.Format("{0}://{1}{2}", HttpContext.Request.Url.Scheme,
                    HttpContext.Request.Url.Authority, "/Browse/Detail/" + model.IdOferta);
                manager.ComprarArticle(model.IdOferta, User.Identity.Name, model.TarjaCredit, model.CodiValidacio, url);
                changeOk = true;
            }
            catch (GaudeixBusinessException ex)
            {
                changeOk = false;
                error = ex.Message.Replace("\r\n", "<br />");
            }

            JsonResult result = new JsonResult();
            result.Data = new { returnValue = changeOk, errorMessage = error };
            Logger.Info("FI - POST: /Browse/BuyPopUp/");
            return result;
        }

        // GET: /Browse/CommentPopUp
        [Authorize]
        public PartialViewResult CommentPopUp(int id)
        {
            Logger.Info("INICI - GET: /Browse/CommentPopUp/" + id.ToString());
            CommentPopUpModel model = new CommentPopUpModel();
            try
            {
                IOfertaManager manager = this.GetManager<IOfertaManager>();
                DetallOfertaModel oferta = manager.GetDetallOfertaForWeb(id);
                model.IdOferta = id;
                model.Titol = oferta.Titol;
                model.Valoracio = 5;
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - GET: /Browse/CommentPopUp/" + id.ToString());
            return PartialView(model);
        }

        // POST: /Browse/CommentPopUp
        [HttpPost]
        [Authorize]
        public JsonResult CommentPopUp(CommentPopUpModel model)
        {
            Logger.Info("INICI - POST: /Browse/CommentPopUp/");
            bool changeOk = false;
            string error = String.Empty;
            try
            {
                IComentariOfertaManager manager = this.GetManager<IComentariOfertaManager>();
                manager.AfegirComentari(model.IdOferta, User.Identity.Name, model.Comentari, model.Valoracio);
                changeOk = true;
            }
            catch (GaudeixBusinessException ex)
            {
                changeOk = false;
                error = ex.Message.Replace("\r\n", "<br/>");
            }
            JsonResult result = new JsonResult();
            result.Data = new { returnValue = changeOk, errorMessage = error };
            Logger.Info("FI - POST: /Browse/CommentPopUp/");
            return result;
        }

        // GET: /Browse/RecommendPopUp
        public PartialViewResult RecommendPopUp(int id)
        {
            Logger.Info("INICI - GET: /Browse/RecommendPopUp/" + id.ToString());
            RecommendPopUpModel model = new RecommendPopUpModel();
            try
            {
                IOfertaManager manager = this.GetManager<IOfertaManager>();
                DetallOfertaModel oferta = manager.GetDetallOfertaForWeb(id);
                model.IdOferta = id;
                model.Titol = oferta.Titol;

                if (User.Identity.IsAuthenticated)
                {
                    IUsuariManager userManager = this.GetManager<IUsuariManager>();
                    model.NomPropi = userManager.GetDetallUsuariByUserName(User.Identity.Name).Nom;
                }
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - GET: /Browse/RecommendPopUp/" + id.ToString());

            return PartialView(model);
        }

        // POST: /Browse/RecommendPopUp
        [HttpPost]
        public JsonResult RecommendPopUp(RecommendPopUpModel model)
        {
            Logger.Info("INICI - POST: /Browse/RecommendPopUp/");
            bool recommendOk = false;
            string error = String.Empty;
            try
            {
                ICuaCorreuManager manager = this.GetManager<ICuaCorreuManager>();

                string url = String.Format("{0}://{1}{2}", HttpContext.Request.Url.Scheme, HttpContext.Request.Url.Authority, "/Browse/Detail/" + model.IdOferta);
                manager.EnqueueMailRecommend(model, url);
                recommendOk = true;
            }
            catch (GaudeixBusinessException ex)
            {
                recommendOk = false;
                Logger.Error("Error al recomanar", ex);
                error = ex.Message.Replace("\r\n", "<br/>");
            }
            JsonResult result = new JsonResult();
            result.Data = new { returnValue = recommendOk, errorMessage = error };
            Logger.Info("FI- POST: /Browse/RecommendPopUp/");
            return result;
        }
        #endregion
    }
}
