﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using System.Web.Security;
using Gaudeix.Common.Exceptions;
using Gaudeix.Common.Interfaces.Manager;
using Gaudeix.Common.Models.Account;
using Gaudeix.Common.Models.Profile;
using Gaudeix.Common.Models.Users;
using Gaudeix.Web.Infrastructure.Controllers;
using Gaudeix.Web.Resources;
using Gaudeix.Web.Security;

namespace Gaudeix.Web.Controllers
{
    /// <summary>
    /// Controlador d'operacions sobre el perfil d'usuari
    /// </summary>
    public class ProfileController : BaseController
    {
        #region Actions del controller
        // GET: /Profile/Detail/id
        [Authorize]
        public ActionResult Detail()
        {
            Logger.Info("INICI - GET: /Profile/Detail/");
            //Carreguem el detall d'usuari
            ProfileModel model = new ProfileModel();
            try
            {
                // Recuperem les dades de l'usuari
                IUsuariManager manager = this.GetManager<IUsuariManager>();
                DetallUsuari usuari = manager.GetDetallUsuariByUserName(User.Identity.Name);

                model = GetProfileModelFromDetallUsuari(usuari);
                ITaulesBaseManager tbManager = this.GetManager<ITaulesBaseManager>();
                // Carreguem la llista de provincies
                model.LlistaProvincies = tbManager.GetProvincies();
                // Carreguem la llista de poblacions
                if (model.idProvincia > 0)
                {
                    model.LlistaPoblacions = tbManager.GetPoblacionsByIdProvincia(model.idProvincia);
                }
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.InnerException);
            }
            Logger.Info("FI - GET: /Profile/Detail/");
            return View(model);
        }

        // POST: /Profile/Detail
        [Authorize]
        [HttpPost]
        public ActionResult Detail(ProfileModel model)
        {
            // Guardem les dades d'usuari a la BD
            Logger.Info("INICI - POST: /Profile/Detail/");
            try
            {
                IUsuariManager manager = this.GetManager<IUsuariManager>();
                DetallUsuari usuari = manager.GetDetallUsuariByUserName(User.Identity.Name);
                usuari.Nom = model.Nom;
                usuari.Cognom1 = model.Cognom1;
                usuari.Cognom2 = model.Cognom2;
                usuari.Nif = model.Nif;
                usuari.Email = model.Email;
                usuari.Adreca = model.Adreca;
                usuari.IdPoblacio = model.idPoblacio;
                usuari.CodiPostal = model.CodiPostal;
                usuari.DataNaixement = model.DataNaixement;
                usuari.LoginUsuari = User.Identity.Name;
                usuari.PreguntaClau = model.PreguntaClau;
                usuari.RespostaClau = model.RespostaClau;
                usuari.MailOfertes = model.MailOfertes;
                usuari.LlistaCategories = model.LlistaCategories;
                // Cridem al manager per a guardar els canvis            
                manager.GuardarUsuari(usuari, false);
                model.InfoMessage = GlobalMessages.InfoDadesGuardades;
                model.UsuariLogin = User.Identity.Name;

                ITaulesBaseManager tbManager = GetManager<ITaulesBaseManager>();
                // Carreguem la llista de provincies
                model.LlistaProvincies = tbManager.GetProvincies();
                // Carreguem la llista de poblacions
                if (model.idProvincia > 0)
                {
                    model.LlistaPoblacions = tbManager.GetPoblacionsByIdProvincia(model.idProvincia);
                }
            }
            catch (GaudeixBusinessException ex)
            {
                ModelState.AddModelError("", ex.Message);
            }
            Logger.Info("FI - POST: /Profile/Detail/");
            return View(model);
        }


        // GET: /Account/ChangePassword
        [Authorize]
        public PartialViewResult ChangePassword()
        {
            Logger.Info("INICI - GET: /Profile/ChangePassword/");
            ChangePasswordModel model = new ChangePasswordModel();
            Logger.Info("FI - GET: /Profile/ChangePassword/");
            return PartialView(model);
        }

        // POST: /Account/ChangePassword
        [Authorize]
        [HttpPost]
        public JsonResult ChangePassword(ChangePasswordModel model)
        {
            Logger.Info("INICI - POST: /Profile/ChangePassword/");
            bool changeOk = false;
            string error = String.Empty;

            try
            {
                changeOk = ((GaudeixMembershipProvider)Membership.Provider).ChangePassword(User.Identity.Name, model.OldPassword, model.NewPassword, model.ConfirmPassword);
            }
            catch (GaudeixMembershipException ex)
            {
                changeOk = false;
                error = ex.Message.Replace("\r\n", "<br />");
            }

            JsonResult result = new JsonResult();
            result.Data = new { returnValue = changeOk, errorMessage = error };
            Logger.Info("FI - POST: /Profile/ChangePassword/");
            return result;
        }
        #endregion

        #region Mètodes privats del controller
        /// <summary>
        /// Obté una classe 'ProfileModel' a partir d'una classe 'DetallUsuari'
        /// </summary>
        /// <param name="usuari"></param>
        /// <returns></returns>
        private ProfileModel GetProfileModelFromDetallUsuari(DetallUsuari usuari)
        {
            ProfileModel model = new ProfileModel();
            // Carreguem les dades amb el resultat
            model.Nom = usuari.Nom;
            model.Cognom1 = usuari.Cognom1;
            model.Cognom2 = usuari.Cognom2;
            model.Nif = usuari.Nif;
            model.Email = usuari.Email;
            model.Adreca = usuari.Adreca;
            model.idProvincia = usuari.IdProvincia;
            model.idPoblacio = usuari.IdPoblacio;
            model.CodiPostal = usuari.CodiPostal;
            model.DataNaixement = usuari.DataNaixement;
            model.UsuariLogin = usuari.LoginUsuari;
            // No carreguem passwords
            model.PreguntaClau = usuari.PreguntaClau;
            model.RespostaClau = usuari.RespostaClau;
            model.MailOfertes = usuari.MailOfertes;
            // Assignem el llistat de compres
            if (usuari.LlistaCompres == null)
            {
                model.LlistaCompres = new List<ItemCompresUsuari>();
            }
            else
            {
                model.LlistaCompres = usuari.LlistaCompres;
            }
            if (usuari.LlistaCategories == null)
            {
                model.LlistaCategories = new List<ItemLlistaCategories>();
            }
            else
            {
                model.LlistaCategories = usuari.LlistaCategories;
            }
            return model;
        }
        #endregion
    }
}
