﻿using System;
using System.Threading;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using Castle.Windsor;
using Gaudeix.Common.Infrastructure.Installer;
using Gaudeix.Web.Infrastructure.Config;
using Gaudeix.Web.Infrastructure.Factory;
using Gaudeix.Web.Infrastructure.Installers;
using System.Net;

namespace Gaudeix.Web
{
    public class MvcApplication : System.Web.HttpApplication
    {
        #region Variables privadas
        /// <summary>
        /// Contenedor de controladores. Se utiliza para inyectar dependencias
        /// </summary>
        private static IWindsorContainer controllers;

        /// <summary>
        /// Timer que dispara el procés de buidat de la cua de correus electrònics
        /// </summary>
        private static Timer timerProcessMail = null;

        /// <summary>
        /// Temporitzador que dispara el proces d'enviament de correus electrònic amb les darreres ofertes
        /// </summary>
        private static Timer timerProcessOfertes = null;

        /// <summary>
        /// URL base de l'aplicació
        /// </summary>
        private static string BaseAddress;
        #endregion

        #region Mètodes privats
        /// <summary>
        /// Inicialitza el contenidor de controladors
        /// </summary>
        private static void InitControllerContainer()
        {
            // Inicialitzem el contenidor de controllers
            controllers = new WindsorContainer();
            // Instal·lació dels controllers al contenidor
            controllers.Install(new ControllersInstaller());
            // Instal·lació del servei de Log al contenidor
            controllers.Install(new LoggerInstaller());
            // Inicialitzem la factory per a crear els controladors
            IControllerFactory controllerFactory = new WindsorControllerFactory(controllers.Kernel);
            // Modifiquem la factoria de creació de controladors, per a que faci servir la nostra
            ControllerBuilder.Current.SetControllerFactory(controllerFactory);
        }

        /// <summary>
        /// Inicialitza els processos batch, basats en temporitzadors
        /// </summary>
        private void InitBatchProcesses()
        {
            // Recuperar configuració del timer de la cua de correu del fitxer web.config
            TimerConfigSection config = (TimerConfigSection)System.Configuration.ConfigurationManager.GetSection("timerConfig");
            // Inicialitzem el temporitzador que buida la cua de correu
            timerProcessMail = new Timer(TimerProcessMail, null, config.CuaCorreu.RetrasInici * 1000, config.CuaCorreu.Interval * 1000);

            timerProcessOfertes = new Timer(TimerProcessOfertes, null, 10000, 60000);
        }

        /// <summary>
        /// Mètode associat al temporitzador que buida la cua de correu.
        /// </summary>
        /// <param name="state"></param>
        private void TimerProcessMail(object state)
        {
            try
            {
                if (String.IsNullOrEmpty(BaseAddress))
                {
                    // Fins que no hi hagi la primera crida a l'aplicació, no s'haurà establert la propietat AbsoluteUri
                    return;
                }
                // Executem el procés en base a una crida MVC, per tal de mantenir la 
                // transaccionabilitat que ens dona NHibernate
                string sURL = BaseAddress + "Mail/ProcessMail/";

                // Inicialitzem la url
                WebRequest wrMail = WebRequest.Create(sURL);
                
                // Cridem a la URL
                wrMail.GetResponse().GetResponseStream();
            }
            catch (Exception ex)
            {
                string message = ex.Message;
            }
             
        }

        /// <summary>
        /// Mètode associat al temporitzador que envia el correu d'ofertes
        /// </summary>
        /// <param name="state"></param>
        private void TimerProcessOfertes(object state)
        {
            try
            {
                if (String.IsNullOrEmpty(BaseAddress))
                {
                    // Fins que no hi hagi la primera crida a l'aplicació, no s'haurà establert la propietat AbsoluteUri
                    return;
                }
                TimerConfigSection config = (TimerConfigSection)System.Configuration.ConfigurationManager.GetSection("timerConfig");
                if ((DateTime.Now.Hour == (int)config.CorreuOfertes.Hora) && (DateTime.Now.Minute == (int)config.CorreuOfertes.Minut))
                {
                    // Enviem el correu d'ofertes
                    string sURL = BaseAddress + "Mail/ProcessOfertes/";
                    // Inicialitzem la url
                    WebRequest wrMail = WebRequest.Create(sURL);

                    // Cridem a la URL
                    wrMail.GetResponse().GetResponseStream();
                }
            }
            catch (Exception ex)
            {
                string message = ex.Message;
            }


        }
        #endregion

        #region Mètodes sobreescrits de HttpApplication
        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            // Evitar errors al demanar el favicon, degut al controlador de WindsorControllerFactory
            routes.IgnoreRoute("{*favicon}", new { favicon = @"(.*/)?favicon.ico(/.*)?" });

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new { controller = "Home", action = "Index", id = UrlParameter.Optional } // Parameter defaults
            );

        }

        protected void Application_BeginRequest(object sender, EventArgs e)
        {
            if (String.IsNullOrEmpty(BaseAddress))
            {
                string address = "{0}://{1}:{2}{3}";
                Uri url = HttpContext.Current.Request.Url;
                address = string.Format(address, url.Scheme, url.Host, url.Port, HttpContext.Current.Request.ApplicationPath);



                if (!address.EndsWith("/"))
                {
                    address += "/";
                }
                // Setejem la URL base de l'aplicació, però només la primera vegada
                BaseAddress = address;
            }
        }

        protected void Application_Start()
        {
            // Inicialitzem el component log4net
            log4net.Config.XmlConfigurator.Configure();

            AreaRegistration.RegisterAllAreas();

            RegisterGlobalFilters(GlobalFilters.Filters);
            RegisterRoutes(RouteTable.Routes);

            // Inicialitzem el contenidor de controllers
            InitControllerContainer();

            // Inicialitzar processos batch
            InitBatchProcesses();
        }

        protected void Application_End()
        {
            DisposeControllerContainer();
        }

        private void DisposeControllerContainer()
        {
            controllers.Dispose();
        }
        #endregion

    }
}