﻿using System;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Text;
using System.Web.Security;
using Gaudeix.Common.Exceptions;
using Gaudeix.Common.Interfaces.Manager;
using Gaudeix.Common.Models.Account;
using Gaudeix.Common.Models.Users;
using Gaudeix.Common.Resources;
using Gaudeix.Manager.Infrastructure.Factory;

namespace Gaudeix.Web.Security
{
    public class GaudeixMembershipProvider : MembershipProvider
    {
        #region Propietats privades de la classe
        private string pApplicationName;
        private int pMaxInvalidPasswordAttempts;
        private int pPasswordAttemptWindow;
        private int pMinRequiredNonAlphanumericCharacters;
        private int pMinRequiredPasswordLength;
        private string pPasswordStrengthRegularExpression;
        private bool pEnablePasswordReset;
        private bool pEnablePasswordRetrieval;
        private bool pRequiresQuestionAndAnswer;
        private bool pRequiresUniqueEmail;
        private bool pWriteExceptionsToEventLog;
        private MembershipPasswordFormat pPasswordFormat;
        #endregion


        #region Implementació de MembershipProvider
        public override void Initialize(string name, NameValueCollection config)
        {
            // Inicialitzem els valors des del fitxer web.config
            if (config == null)
            {
                throw new ArgumentNullException("config");
            }

            if (name == null || name.Length == 0)
            {
                name = "GaudeixMembershipProvider";
            }

            if (String.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", "Membership provider del projecte Gaudeix");
            }

            // Inicialitzem la classe base abstracta
            base.Initialize(name, config);

            pApplicationName = GetConfigValue(config["applicationName"], System.Web.Hosting.HostingEnvironment.ApplicationVirtualPath);
            pMaxInvalidPasswordAttempts = Convert.ToInt32(GetConfigValue(config["maxInvalidPasswordAttempts"], "5"));
            pPasswordAttemptWindow = Convert.ToInt32(GetConfigValue(config["passwordAttemptWindow"], "10"));
            pMinRequiredNonAlphanumericCharacters = Convert.ToInt32(GetConfigValue(config["minRequiredNonAlphanumericCharacters"], "1"));
            pMinRequiredPasswordLength = Convert.ToInt32(GetConfigValue(config["minRequiredPasswordLength"], "7"));
            pPasswordStrengthRegularExpression = Convert.ToString(GetConfigValue(config["passwordStrengthRegularExpression"], ""));
            pEnablePasswordReset = Convert.ToBoolean(GetConfigValue(config["enablePasswordReset"], "true"));
            pEnablePasswordRetrieval = Convert.ToBoolean(GetConfigValue(config["enablePasswordRetrieval"], "true"));
            pRequiresQuestionAndAnswer = Convert.ToBoolean(GetConfigValue(config["requiresQuestionAndAnswer"], "false"));
            pRequiresUniqueEmail = Convert.ToBoolean(GetConfigValue(config["requiresUniqueEmail"], "true"));
            pWriteExceptionsToEventLog = Convert.ToBoolean(GetConfigValue(config["writeExceptionsToEventLog"], "true"));

            string temp_format = config["passwordFormat"];
            if (temp_format == null)
            {
                temp_format = "Hashed";
            }

            switch (temp_format)
            {
                case "Hashed":
                    pPasswordFormat = MembershipPasswordFormat.Hashed;
                    break;
                case "Encrypted":
                    pPasswordFormat = MembershipPasswordFormat.Encrypted;
                    break;
                case "Clear":
                    pPasswordFormat = MembershipPasswordFormat.Clear;
                    break;
                default:
                    throw new ProviderException("El format de les contrasenyes no està suportat.");
            }
        }

        public override string ApplicationName
        {
            get { return pApplicationName; }
            set { pApplicationName = value; }
        }

        public bool ChangePassword(string username, string oldPassword, string newPassword, string newPasswordConfirm)
        {
            try
            {
                StringBuilder errors = new StringBuilder();
                // Validacions d'integritat
                // Password antic obligatori
                if (String.IsNullOrEmpty(oldPassword))
                {
                    errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.ContrasenyaActual));
                }

                // Password nou obligatori
                if (String.IsNullOrEmpty(newPassword))
                {
                    errors.AppendLine(String.Format(CommonErrors.CampObligatori, CommonFields.NewPassword));
                }
                else
                {
                    if (newPassword.Length < Membership.MinRequiredPasswordLength)
                    {
                        errors.AppendLine(String.Format(CommonErrors.NumCaracters, CommonFields.NewPassword,
                            String.Empty, Membership.MinRequiredPasswordLength));
                    }

                    if (!newPassword.Equals(newPasswordConfirm))
                    {
                        errors.AppendLine(CommonErrors.ParaulesClauDiferents);
                    }
                }
                if (errors.Length > 0)
                {
                    throw new GaudeixBusinessException(errors.ToString());
                }
                IUsuariManager manager = ManagerFactory.GetManager<IUsuariManager>();
                return manager.CanviarContrasenya(username, oldPassword, newPassword, newPasswordConfirm);
            }
            catch (GaudeixBusinessException ex)
            {
                throw new GaudeixMembershipException(ex);
            }
        }

        public override bool ChangePassword(string username, string oldPassword, string newPassword)
        {
            throw new NotImplementedException();
        }

        public override bool ChangePasswordQuestionAndAnswer(string username, string password, string newPasswordQuestion, string newPasswordAnswer)
        {
            throw new NotImplementedException();
        }

        public MembershipUser CreateUser(RegisterModel user, out MembershipCreateStatus createStatus)
        {
            GaudeixMembershipUser result = null;
            try
            {
                IUsuariManager manager = ManagerFactory.GetManager<IUsuariManager>();


                if (manager.ExisteixUsuariByUserName(user.UsuariLogin))
                {
                    createStatus = MembershipCreateStatus.DuplicateUserName;
                }
                else if (manager.ExisteixUsuariByNIF(user.Nif))
                {
                    createStatus = MembershipCreateStatus.DuplicateProviderUserKey;
                }
                else
                {

                    DetallUsuari usuari = new DetallUsuari(0, user.Nom, user.Cognom1, user.Cognom2, user.Nif, user.Email,
                            user.Adreca, String.Empty, string.Empty, user.idProvincia, user.idPoblacio, user.DataNaixement,
                            user.UsuariLogin, user.UsuariPassword, user.PreguntaClau, user.RespostaClau, user.MailOfertes,
                            user.CodiPostal);
                    usuari.LlistaCategories = user.LlistaCategories;
                    usuari.LoginPasswordConfirm = user.ConfirmUsuariPassword;
                    int idUser = manager.GuardarUsuari(usuari, true);
                    DetallUsuari detall = manager.GetDetallUsuariById(idUser);
                    result = new GaudeixMembershipUser(detall.Nom, detall.Cognom1, detall.Cognom2, detall.Nif,
                        detall.Email, detall.Adreca, detall.IdProvincia, detall.IdPoblacio, detall.NomProvincia, detall.NomPoblacio,
                        detall.CodiPostal, detall.DataNaixement, detall.LoginUsuari);
                    createStatus = MembershipCreateStatus.Success;
                }
            }
            catch (GaudeixBusinessException ex)
            {
                throw new GaudeixMembershipException(ex);
            }
            catch (Exception)
            {
                createStatus = MembershipCreateStatus.ProviderError;
            }

            return result;
        }



        public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
        {
            throw new NotImplementedException();
        }

        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            throw new NotImplementedException();
        }

        public override bool EnablePasswordReset
        {
            get { return pEnablePasswordReset; }
        }

        public override bool EnablePasswordRetrieval
        {
            get { return pEnablePasswordRetrieval; }
        }

        public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        public override int GetNumberOfUsersOnline()
        {
            throw new NotImplementedException();
        }

        public override string GetPassword(string username, string answer)
        {
            throw new NotImplementedException();
        }

        public override MembershipUser GetUser(string username, bool userIsOnline)
        {
            throw new NotImplementedException();
        }

        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
            throw new NotImplementedException();
        }

        public override string GetUserNameByEmail(string email)
        {
            throw new NotImplementedException();
        }



        public override string ResetPassword(string username, string answer)
        {
            throw new NotImplementedException();
        }

        public override bool UnlockUser(string userName)
        {
            throw new NotImplementedException();
        }

        public override void UpdateUser(MembershipUser user)
        {
            throw new NotImplementedException();
        }

        public override bool ValidateUser(string username, string password)
        {
            try
            {
                IUsuariManager user = ManagerFactory.GetManager<IUsuariManager>();
                return user.ValidateUser(username, password);
            }
            catch (GaudeixBusinessException)
            {
                // L'excepció ja ha estat loguejada al manager
                return false;
            }
        }
        #endregion

        #region Funcions privades de la classe

        /// <summary>
        /// Funció que recupera valors des del fitxer de configuració
        /// </summary>
        /// <param name="configValue">Clau del valor a recuperar</param>
        /// <param name="defaultValue">Valor per defecte</param>
        /// <returns>Valor de la clau si es troba, o defaultValue si no es troba.</returns>
        private string GetConfigValue(string configValue, string defaultValue)
        {
            if (String.IsNullOrEmpty(configValue))
                return defaultValue;

            return configValue;
        }

        #endregion

        #region Propietats de la classe
        public override int MaxInvalidPasswordAttempts
        {
            get { return pMaxInvalidPasswordAttempts; }
        }

        public override int MinRequiredNonAlphanumericCharacters
        {
            get { return pMinRequiredNonAlphanumericCharacters; }
        }

        public override int MinRequiredPasswordLength
        {
            get { return pMinRequiredPasswordLength; }
        }

        public override int PasswordAttemptWindow
        {
            get { return pPasswordAttemptWindow; }
        }

        public override MembershipPasswordFormat PasswordFormat
        {
            get { return pPasswordFormat; }
        }

        public override string PasswordStrengthRegularExpression
        {
            get { return pPasswordStrengthRegularExpression; }
        }

        public override bool RequiresQuestionAndAnswer
        {
            get { return pRequiresQuestionAndAnswer; }
        }

        public override bool RequiresUniqueEmail
        {
            get { return pRequiresUniqueEmail; }
        }
        #endregion
    }
}